@@ -17,7 +17,7 @@
  *
  * You should have received a copy of the GNU General Public License
  * along with this program; if not, write to the Free Software
- * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
+ * Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
  *
  * Author: David Zeuthen <davidz@redhat.com>
  */
@@ -33,7 +33,6 @@
 #include <gdk/gdkkeysyms.h>
 
 #include <eel/eel-glib-extensions.h>
-#include <eel/eel-stock-dialogs.h>
 
 #include "caja-icon-info.h"
 #include "caja-global-preferences.h"
@@ -44,6 +43,8 @@
 #include "caja-desktop-icon-file.h"
 #include "caja-file-utilities.h"
 
+#include <src/glibcompat.h> /* for g_list_free_full */
+
 enum
 {
     AUTORUN_ASK,
@@ -83,9 +84,9 @@ caja_autorun_get_preferences (const char *x_content_type,
     *pref_start_app = FALSE;
     *pref_ignore = FALSE;
     *pref_open_folder = FALSE;
-    x_content_start_app = eel_preferences_get_string_array (CAJA_PREFERENCES_MEDIA_AUTORUN_X_CONTENT_START_APP);
-    x_content_ignore = eel_preferences_get_string_array (CAJA_PREFERENCES_MEDIA_AUTORUN_X_CONTENT_IGNORE);
-    x_content_open_folder = eel_preferences_get_string_array (CAJA_PREFERENCES_MEDIA_AUTORUN_X_CONTENT_OPEN_FOLDER);
+    x_content_start_app = g_settings_get_strv (caja_media_preferences, CAJA_PREFERENCES_MEDIA_AUTORUN_X_CONTENT_START_APP);
+    x_content_ignore = g_settings_get_strv (caja_media_preferences, CAJA_PREFERENCES_MEDIA_AUTORUN_X_CONTENT_IGNORE);
+    x_content_open_folder = g_settings_get_strv (caja_media_preferences, CAJA_PREFERENCES_MEDIA_AUTORUN_X_CONTENT_OPEN_FOLDER);
     if (x_content_start_app != NULL)
     {
         *pref_start_app = eel_g_strv_find (x_content_start_app, x_content_type) != -1;
@@ -156,30 +157,30 @@ caja_autorun_set_preferences (const char *x_content_type,
 
     g_assert (x_content_type != NULL);
 
-    x_content_start_app = eel_preferences_get_string_array (CAJA_PREFERENCES_MEDIA_AUTORUN_X_CONTENT_START_APP);
-    x_content_ignore = eel_preferences_get_string_array (CAJA_PREFERENCES_MEDIA_AUTORUN_X_CONTENT_IGNORE);
-    x_content_open_folder = eel_preferences_get_string_array (CAJA_PREFERENCES_MEDIA_AUTORUN_X_CONTENT_OPEN_FOLDER);
+    x_content_start_app = g_settings_get_strv (caja_media_preferences, CAJA_PREFERENCES_MEDIA_AUTORUN_X_CONTENT_START_APP);
+    x_content_ignore = g_settings_get_strv (caja_media_preferences, CAJA_PREFERENCES_MEDIA_AUTORUN_X_CONTENT_IGNORE);
+    x_content_open_folder = g_settings_get_strv (caja_media_preferences, CAJA_PREFERENCES_MEDIA_AUTORUN_X_CONTENT_OPEN_FOLDER);
 
     remove_elem_from_str_array (x_content_start_app, x_content_type);
     if (pref_start_app)
     {
         x_content_start_app = add_elem_to_str_array (x_content_start_app, x_content_type);
     }
-    eel_preferences_set_string_array (CAJA_PREFERENCES_MEDIA_AUTORUN_X_CONTENT_START_APP, x_content_start_app);
+    g_settings_set_strv (caja_media_preferences, CAJA_PREFERENCES_MEDIA_AUTORUN_X_CONTENT_START_APP, (const gchar * const*) x_content_start_app);
 
     remove_elem_from_str_array (x_content_ignore, x_content_type);
     if (pref_ignore)
     {
         x_content_ignore = add_elem_to_str_array (x_content_ignore, x_content_type);
     }
-    eel_preferences_set_string_array (CAJA_PREFERENCES_MEDIA_AUTORUN_X_CONTENT_IGNORE, x_content_ignore);
+    g_settings_set_strv (caja_media_preferences, CAJA_PREFERENCES_MEDIA_AUTORUN_X_CONTENT_IGNORE, (const gchar * const*) x_content_ignore);
 
     remove_elem_from_str_array (x_content_open_folder, x_content_type);
     if (pref_open_folder)
     {
         x_content_open_folder = add_elem_to_str_array (x_content_open_folder, x_content_type);
     }
-    eel_preferences_set_string_array (CAJA_PREFERENCES_MEDIA_AUTORUN_X_CONTENT_OPEN_FOLDER, x_content_open_folder);
+    g_settings_set_strv (caja_media_preferences, CAJA_PREFERENCES_MEDIA_AUTORUN_X_CONTENT_OPEN_FOLDER, (const gchar * const*) x_content_open_folder);
 
     g_strfreev (x_content_open_folder);
     g_strfreev (x_content_ignore);
@@ -274,7 +275,7 @@ dialog_response_cb (GtkDialog *dialog,
 }
 
 static void
-dialog_destroy_cb (GtkObject *object,
+dialog_destroy_cb (GtkWidget *object,
                    CajaAutorunComboBoxData *data)
 {
     handle_dialog_closure (data);
@@ -652,7 +653,7 @@ caja_autorun_prepare_combo_box (GtkWidget *combo_box,
     {
         g_object_unref (default_app_info);
     }
-    eel_g_object_list_free (app_info_list);
+    g_list_free_full (app_info_list, g_object_unref);
 
     gtk_combo_box_set_model (GTK_COMBO_BOX (combo_box), GTK_TREE_MODEL (list_store));
     g_object_unref (G_OBJECT (list_store));
@@ -747,8 +748,13 @@ is_shift_pressed (void)
     ret = FALSE;
 
     gdk_error_trap_push ();
-    status = XkbGetState (GDK_DISPLAY (), XkbUseCoreKbd, &state);
+    status = XkbGetState (GDK_DISPLAY_XDISPLAY (gdk_display_get_default ()),
+    			  XkbUseCoreKbd, &state);
+#if GTK_CHECK_VERSION(3,0,0)
+    gdk_error_trap_pop_ignored ();
+#else
     gdk_error_trap_pop ();
+#endif
 
     if (status == Success)
     {
@@ -907,7 +913,7 @@ autorun_always_toggled (GtkToggleButton *togglebutton, AutorunDialogData *data)
 static gboolean
 combo_box_enter_ok (GtkWidget *togglebutton, GdkEventKey *event, GtkDialog *dialog)
 {
-    if (event->keyval == GDK_KP_Enter || event->keyval == GDK_Return)
+    if (event->keyval == GDK_KEY_KP_Enter || event->keyval == GDK_KEY_Return)
     {
         gtk_dialog_response (dialog, GTK_RESPONSE_OK);
         return TRUE;
@@ -992,7 +998,6 @@ show_dialog:
 
     dialog = gtk_dialog_new ();
 
-    gtk_dialog_set_has_separator (GTK_DIALOG (dialog), FALSE);
     hbox = gtk_hbox_new (FALSE, 12);
     gtk_box_pack_start (GTK_BOX (gtk_dialog_get_content_area (GTK_DIALOG (dialog))), hbox, TRUE, TRUE, 0);
     gtk_container_set_border_width (GTK_CONTAINER (hbox), 12);
@@ -1217,7 +1222,7 @@ autorun_guessed_content_type_callback (GObject *source_object,
         }
         else
         {
-            if (eel_preferences_get_boolean (CAJA_PREFERENCES_MEDIA_AUTOMOUNT_OPEN))
+            if (g_settings_get_boolean (caja_media_preferences, CAJA_PREFERENCES_MEDIA_AUTOMOUNT_OPEN))
                 open_folder = TRUE;
         }
     }
@@ -1238,7 +1243,7 @@ caja_autorun (GMount *mount, CajaAutorunOpenWindow open_window_func, gpointer us
     AutorunData *data;
 
     if (!should_autorun_mount (mount) ||
-            eel_preferences_get_boolean (CAJA_PREFERENCES_MEDIA_AUTORUN_NEVER))
+            g_settings_get_boolean (caja_media_preferences, CAJA_PREFERENCES_MEDIA_AUTORUN_NEVER))
     {
         return;
     }
