@@ -16,8 +16,8 @@
 
    You should have received a copy of the GNU General Public
    License along with this program; if not, write to the
-   Free Software Foundation, Inc., 59 Temple Place - Suite 330,
-   Boston, MA 02111-1307, USA.
+   Free Software Foundation, Inc., 51 Franklin St, Fifth Floor,
+   Boston, MA 02110-1301, USA.
 
    Author: Federico Mena-Quintero <federico@novell.com>
 */
@@ -27,10 +27,11 @@
 #include <string.h>
 #include <time.h>
 #include <sys/time.h>
-#include <eel/eel-glib-extensions.h>
 #include "caja-debug-log.h"
 #include "caja-file.h"
 
+#include <src/glibcompat.h> /* for g_list_free_full */
+
 #define DEFAULT_RING_BUFFER_NUM_LINES 1000
 
 #define KEY_FILE_GROUP		"debug log"
@@ -39,7 +40,11 @@
 
 #define MAX_URI_COUNT 20
 
+#if GLIB_CHECK_VERSION(2, 31, 0)
+static GMutex log_mutex;
+#else
 static GStaticMutex log_mutex = G_STATIC_MUTEX_INIT;
+#endif
 
 static GHashTable *domains_hash;
 static char **ring_buffer;
@@ -53,13 +58,21 @@ static GSList *milestones_tail;
 static void
 lock (void)
 {
+    #if GLIB_CHECK_VERSION(2, 31, 0)
+    g_mutex_lock (&log_mutex);
+    #else
     g_static_mutex_lock (&log_mutex);
+    #endif
 }
 
 static void
 unlock (void)
 {
+    #if GLIB_CHECK_VERSION(2, 31, 0)
+    g_mutex_unlock (&log_mutex);
+    #else
     g_static_mutex_unlock (&log_mutex);
+    #endif
 }
 
 void
@@ -309,7 +322,7 @@ caja_debug_log_with_file_list (gboolean is_milestone, const char *domain, GList
     caja_debug_logv (is_milestone, domain, uris, format, args);
     va_end (args);
 
-    eel_g_list_free_deep (uris);
+    g_list_free_full (uris, g_free);
 }
 
 gboolean
