@@ -16,8 +16,8 @@
 
    You should have received a copy of the GNU General Public
    License along with this program; if not, write to the
-   Free Software Foundation, Inc., 59 Temple Place - Suite 330,
-   Boston, MA 02111-1307, USA.
+   Free Software Foundation, Inc., 51 Franklin St, Fifth Floor,
+   Boston, MA 02110-1301, USA.
 
    Author: Darin Adler <darin@bentspoon.com>
 */
@@ -32,14 +32,15 @@
 #include "caja-search-directory.h"
 #include "caja-global-preferences.h"
 #include "caja-lib-self-check-functions.h"
-#include "caja-marshal.h"
 #include "caja-metadata.h"
 #include "caja-desktop-directory.h"
 #include "caja-vfs-directory.h"
 #include <eel/eel-glib-extensions.h>
 #include <eel/eel-gtk-macros.h>
-#include <eel/eel-string.h>
 #include <gtk/gtk.h>
+#include <string.h>
+
+#include <src/glibcompat.h> /* for g_list_free_full */
 
 enum
 {
@@ -176,7 +177,7 @@ caja_directory_finalize (GObject *object)
     if (directory->details->monitor_list != NULL)
     {
         g_warning ("destroying a CajaDirectory while it's being monitored");
-        eel_g_list_free_deep (directory->details->monitor_list);
+        g_list_free_full (directory->details->monitor_list, g_free);
     }
 
     if (directory->details->monitor != NULL)
@@ -213,7 +214,7 @@ caja_directory_finalize (GObject *object)
     g_assert (directory->details->directory_load_in_progress == NULL);
     g_assert (directory->details->count_in_progress == NULL);
     g_assert (directory->details->dequeue_pending_idle_id == 0);
-    eel_g_object_list_free (directory->details->pending_file_info);
+    g_list_free_full (directory->details->pending_file_info, g_object_unref);
 
     EEL_CALL_PARENT (G_OBJECT_CLASS, finalize, (object));
 }
@@ -265,9 +266,7 @@ collect_all_directories (gpointer key, gpointer value, gpointer callback_data)
 {
     CajaDirectory *directory;
     GList **dirs;
-    GFile *location;
 
-    location = (GFile *) key;
     directory = CAJA_DIRECTORY (value);
     dirs = callback_data;
 
@@ -327,21 +326,22 @@ add_preferences_callbacks (void)
 {
     caja_global_preferences_init ();
 
-    eel_preferences_add_callback (CAJA_PREFERENCES_SHOW_HIDDEN_FILES,
-                                  filtering_changed_callback,
-                                  NULL);
-    eel_preferences_add_callback (CAJA_PREFERENCES_SHOW_BACKUP_FILES,
-                                  filtering_changed_callback,
-                                  NULL);
-    eel_preferences_add_callback (CAJA_PREFERENCES_SHOW_TEXT_IN_ICONS,
-                                  async_data_preference_changed_callback,
-                                  NULL);
-    eel_preferences_add_callback (CAJA_PREFERENCES_SHOW_DIRECTORY_ITEM_COUNTS,
-                                  async_data_preference_changed_callback,
-                                  NULL);
-    eel_preferences_add_callback (CAJA_PREFERENCES_DATE_FORMAT,
-                                  async_data_preference_changed_callback,
-                                  NULL);
+    g_signal_connect_swapped (caja_preferences,
+                              "changed::" CAJA_PREFERENCES_SHOW_HIDDEN_FILES,
+                              G_CALLBACK(filtering_changed_callback),
+                              NULL);
+    g_signal_connect_swapped (caja_preferences,
+                              "changed::" CAJA_PREFERENCES_SHOW_TEXT_IN_ICONS,
+                              G_CALLBACK (async_data_preference_changed_callback),
+                              NULL);
+    g_signal_connect_swapped (caja_preferences,
+                              "changed::" CAJA_PREFERENCES_SHOW_DIRECTORY_ITEM_COUNTS,
+                              G_CALLBACK (async_data_preference_changed_callback),
+                              NULL);
+    g_signal_connect_swapped (caja_preferences,
+                              "changed::" CAJA_PREFERENCES_DATE_FORMAT,
+                              G_CALLBACK(async_data_preference_changed_callback),
+                              NULL);
 }
 
 /**
@@ -717,33 +717,6 @@ caja_directory_remove_file (CajaDirectory *directory, CajaFile *file)
     }
 }
 
-#define CAJA_DIRECTORY_FILE_LIST_DEFAULT_LIMIT 4000
-
-gboolean
-caja_directory_file_list_length_reached (CajaDirectory *directory)
-{
-    static gboolean inited = FALSE;
-    static int directory_limit = 0;
-
-    if (!inited)
-    {
-        eel_preferences_add_auto_integer
-        ("/apps/caja/preferences/directory_limit",
-         &directory_limit);
-        inited = TRUE;
-    }
-    if (directory_limit < 0)    /* unlimited */
-    {
-        return FALSE;
-    }
-    if (directory_limit == 0)   /* dead mateconfd */
-    {
-        directory_limit = CAJA_DIRECTORY_FILE_LIST_DEFAULT_LIMIT;
-    }
-
-    return directory->details->confirmed_file_count >= directory_limit;
-}
-
 GList *
 caja_directory_begin_file_name_change (CajaDirectory *directory,
                                        CajaFile *file)
@@ -785,7 +758,7 @@ caja_directory_find_file_by_internal_filename (CajaDirectory *directory,
 {
     CajaFile *result;
 
-    if (eel_strcmp (internal_filename, ".") == 0)
+    if (g_strcmp0 (internal_filename, ".") == 0)
     {
         result = caja_directory_get_existing_corresponding_file (directory);
         if (result != NULL)
@@ -966,8 +939,7 @@ call_get_file_info_free_list (gpointer key, gpointer value, gpointer user_data)
     files = value;
 
     caja_directory_get_info_for_new_files (directory, files);
-    g_list_foreach (files, (GFunc) g_object_unref, NULL);
-    g_list_free (files);
+    g_list_free_full (files, g_object_unref);
 }
 
 static void
@@ -1119,7 +1091,7 @@ caja_directory_notify_files_added_by_uri (GList *uris)
 
     files = caja_file_list_from_uris (uris);
     caja_directory_notify_files_added (files);
-    eel_g_object_list_free (files);
+    g_list_free_full (files, g_object_unref);
 }
 
 void
@@ -1168,7 +1140,7 @@ caja_directory_notify_files_changed_by_uri (GList *uris)
 
     files = caja_file_list_from_uris (uris);
     caja_directory_notify_files_changed (files);
-    eel_g_object_list_free (files);
+    g_list_free_full (files, g_object_unref);
 }
 
 void
@@ -1229,7 +1201,7 @@ caja_directory_notify_files_removed_by_uri (GList *uris)
 
     files = caja_file_list_from_uris (uris);
     caja_directory_notify_files_changed (files);
-    eel_g_object_list_free (files);
+    g_list_free_full (files, g_object_unref);
 }
 
 static void
@@ -1522,8 +1494,7 @@ caja_directory_notify_files_moved_by_uri (GList *uri_pairs)
 
     file_pairs = uri_pairs_to_file_pairs (uri_pairs);
     caja_directory_notify_files_moved (file_pairs);
-    g_list_foreach (file_pairs, (GFunc)g_file_pair_free, NULL);
-    g_list_free (file_pairs);
+    g_list_free_full (file_pairs, g_file_pair_free);
 }
 
 void
@@ -1659,7 +1630,6 @@ void
 caja_directory_file_monitor_add (CajaDirectory *directory,
                                  gconstpointer client,
                                  gboolean monitor_hidden_files,
-                                 gboolean monitor_backup_files,
                                  CajaFileAttributes file_attributes,
                                  CajaDirectoryCallback callback,
                                  gpointer callback_data)
@@ -1671,7 +1641,6 @@ caja_directory_file_monitor_add (CajaDirectory *directory,
     (CAJA_DIRECTORY_CLASS, directory,
      file_monitor_add, (directory, client,
                         monitor_hidden_files,
-                        monitor_backup_files,
                         file_attributes,
                         callback, callback_data));
 }
@@ -1926,7 +1895,7 @@ caja_self_check_directory (void)
 
     caja_directory_file_monitor_add
     (directory, &data_dummy,
-     TRUE, TRUE, 0, NULL, NULL);
+     TRUE, 0, NULL, NULL);
 
     /* FIXME: these need to be updated to the new metadata infrastructure
      *  as make check doesn't pass.
