@@ -17,8 +17,8 @@
 
    You should have received a copy of the GNU Library General Public
    License along with the Mate Library; see the file COPYING.LIB.  If not,
-   write to the Free Software Foundation, Inc., 59 Temple Place - Suite 330,
-   Boston, MA 02111-1307, USA.
+   write to the Free Software Foundation, Inc., 51 Franklin St, Fifth Floor,
+   Boston, MA 02110-1301, USA.
 
    Authors: Pavel Cisler <pavel@eazel.com>,
    	    Ettore Perazzoli <ettore@gnu.org>
@@ -44,6 +44,8 @@
 #include <stdio.h>
 #include <string.h>
 
+#include <src/glibcompat.h> /* for g_list_free_full */
+
 /* a set of defines stolen from the eel-icon-dnd.c file.
  * These are in microseconds.
  */
@@ -129,7 +131,7 @@ caja_drag_uri_array_from_selection_list (const GList *selection_list)
 
     uri_list = caja_drag_uri_list_from_selection_list (selection_list);
     uris = caja_drag_uri_array_from_list (uri_list);
-    eel_g_list_free_deep (uri_list);
+    g_list_free_full (uri_list, g_free);
 
     return uris;
 }
@@ -1038,7 +1040,7 @@ caja_drag_autoscroll_calculate_delta (GtkWidget *widget, float *x_scroll_delta,
 void
 caja_drag_autoscroll_start (CajaDragInfo *drag_info,
                             GtkWidget        *widget,
-                            GtkFunction       callback,
+                            GSourceFunc       callback,
                             gpointer          user_data)
 {
     if (caja_drag_autoscroll_in_scroll_region (widget))
@@ -1315,7 +1317,7 @@ slot_proxy_handle_drop (GtkWidget                *widget,
                                                 uri_list,
                                                 target_uri,
                                                 gdk_drag_context_get_selected_action (context));
-            eel_g_list_free_deep (uri_list);
+            g_list_free_full (uri_list, g_free);
         }
         else if (drag_info->info == CAJA_ICON_DND_URI_LIST)
         {
