@@ -16,8 +16,8 @@
 
    You should have received a copy of the GNU Library General Public
    License along with the Mate Library; see the file COPYING.LIB.  If not,
-   write to the Free Software Foundation, Inc., 59 Temple Place - Suite 330,
-   Boston, MA 02111-1307, USA.
+   write to the Free Software Foundation, Inc., 51 Franklin St, Fifth Floor,
+   Boston, MA 02110-1301, USA.
 
    Authors: John Sullivan <sullivan@eazel.com>
 */
@@ -43,6 +43,8 @@
 #include <unistd.h>
 #include <stdlib.h>
 
+#include <src/glibcompat.h> /* for g_list_free_full */
+
 #define CAJA_USER_DIRECTORY_NAME ".config/caja"
 #define DEFAULT_CAJA_DIRECTORY_MODE (0755)
 
@@ -510,7 +512,7 @@ caja_get_xdg_dir (const char *type)
 static char *
 get_desktop_path (void)
 {
-    if (eel_preferences_get_boolean (CAJA_PREFERENCES_DESKTOP_IS_HOME_DIR))
+    if (g_settings_get_boolean (caja_preferences, CAJA_PREFERENCES_DESKTOP_IS_HOME_DIR))
     {
         return g_strdup (g_get_home_dir());
     }
@@ -535,7 +537,7 @@ caja_get_desktop_directory (void)
     desktop_directory = get_desktop_path ();
 
     /* Don't try to create a home directory */
-    if (!eel_preferences_get_boolean (CAJA_PREFERENCES_DESKTOP_IS_HOME_DIR))
+    if (!g_settings_get_boolean (caja_preferences, CAJA_PREFERENCES_DESKTOP_IS_HOME_DIR))
     {
         if (!g_file_test (desktop_directory, G_FILE_TEST_EXISTS))
         {
@@ -765,9 +767,9 @@ caja_is_desktop_directory_file (GFile *dir,
 
     if (!desktop_dir_changed_callback_installed)
     {
-        eel_preferences_add_callback (CAJA_PREFERENCES_DESKTOP_IS_HOME_DIR,
-                                      desktop_dir_changed_callback,
-                                      NULL);
+        g_signal_connect_swapped (caja_preferences, "changed::" CAJA_PREFERENCES_DESKTOP_IS_HOME_DIR,
+                                  G_CALLBACK(desktop_dir_changed_callback),
+                                  NULL);
         desktop_dir_changed_callback_installed = TRUE;
     }
 
@@ -786,9 +788,9 @@ caja_is_desktop_directory (GFile *dir)
 
     if (!desktop_dir_changed_callback_installed)
     {
-        eel_preferences_add_callback (CAJA_PREFERENCES_DESKTOP_IS_HOME_DIR,
-                                      desktop_dir_changed_callback,
-                                      NULL);
+        g_signal_connect_swapped (caja_preferences, "changed::" CAJA_PREFERENCES_DESKTOP_IS_HOME_DIR,
+                                  G_CALLBACK(desktop_dir_changed_callback),
+                                  NULL);
         desktop_dir_changed_callback_installed = TRUE;
     }
 
@@ -1428,7 +1430,7 @@ caja_restore_files_from_trash (GList *files,
              parent_window,
              NULL, NULL);
 
-            eel_g_object_list_free (locations);
+            g_list_free_full (locations, g_object_unref);
             g_object_unref (original_dir_location);
         }
 
