@@ -16,8 +16,8 @@
 
    You should have received a copy of the GNU General Public
    License along with this program; if not, write to the
-   Free Software Foundation, Inc., 59 Temple Place - Suite 330,
-   Boston, MA 02111-1307, USA.
+   Free Software Foundation, Inc., 51 Franklin St, Fifth Floor,
+   Boston, MA 02110-1301, USA.
 
    Author: Darin Adler <darin@bentspoon.com>
 */
@@ -425,14 +425,11 @@ int                     caja_file_compare_location                  (CajaFile
 
 /* filtering functions for use by various directory views */
 gboolean                caja_file_is_hidden_file                    (CajaFile                   *file);
-gboolean                caja_file_is_backup_file                    (CajaFile                   *file);
 gboolean                caja_file_should_show                       (CajaFile                   *file,
         gboolean                        show_hidden,
-        gboolean                        show_backup,
         gboolean                        show_foreign);
-GList                  *caja_file_list_filter_hidden_and_backup     (GList                          *files,
-        gboolean                        show_hidden,
-        gboolean                        show_backup);
+GList                  *caja_file_list_filter_hidden                (GList                          *files,
+        gboolean                        show_hidden);
 
 
 /* Get the URI that's used when activating the file.
