@@ -20,8 +20,8 @@
  *
  * You should have received a copy of the GNU General Public
  * License along with this library; if not, write to the
- * Free Software Foundation, Inc., 59 Temple Place - Suite 330,
- * Boston, MA 02111-1307, USA.
+ * Free Software Foundation, Inc., 51 Franklin St, Fifth Floor,
+ * Boston, MA 02110-1301, USA.
  */
 
 #include "caja-undostack-manager.h"
@@ -33,7 +33,8 @@
 #include <glib/gi18n.h>
 #include <locale.h>
 #include <gdk/gdk.h>
-#include <eel/eel-glib-extensions.h>
+
+#include <src/glibcompat.h> /* for g_list_free_full */
 
 /* *****************************************************************
  Private fields
@@ -99,7 +100,11 @@ struct _CajaUndoStackManagerPrivate
   GQueue *stack;
   guint undo_levels;
   guint index;
+#if GLIB_CHECK_VERSION(2, 32, 0)
+  GMutex mutex;                /* Used to protect access to stack (because of async file ops) */
+#else
   GMutex *mutex;                /* Used to protect access to stack (because of async file ops) */
+#endif
   gboolean dispose_has_run;
   gboolean undo_redo_flag;
   gboolean confirm_delete;
@@ -268,7 +273,11 @@ caja_undostack_manager_init (CajaUndoStackManager * self)
 
   /* Initialize private fields */
   priv->stack = g_queue_new ();
+#if GLIB_CHECK_VERSION(2, 32, 0)
+  g_mutex_init (&priv->mutex);
+#else
   priv->mutex = g_mutex_new ();
+#endif
   priv->index = 0;
   priv->dispose_has_run = FALSE;
   priv->undo_redo_flag = FALSE;
@@ -284,14 +293,22 @@ caja_undostack_manager_dispose (GObject * object)
   if (priv->dispose_has_run)
     return;
 
+#if GLIB_CHECK_VERSION(2, 32, 0)
+  g_mutex_lock (&priv->mutex);
+#else
   g_mutex_lock (priv->mutex);
+#endif
 
   /* Free each undoable action in the stack and the stack itself */
   undostack_dispose_all (priv->stack);
   g_queue_free (priv->stack);
+#if GLIB_CHECK_VERSION(2, 32, 0)
+  g_mutex_unlock (&priv->mutex);
+  g_mutex_clear (&priv->mutex);
+#else
   g_mutex_unlock (priv->mutex);
-
   g_mutex_free (priv->mutex);
+#endif
 
   priv->dispose_has_run = TRUE;
 
@@ -322,9 +339,17 @@ caja_undostack_manager_set_property (GObject * object, guint prop_id,
       new_undo_levels = g_value_get_uint (value);
       if (new_undo_levels > 0 && (priv->undo_levels != new_undo_levels)) {
         priv->undo_levels = new_undo_levels;
+#if GLIB_CHECK_VERSION(2, 32, 0)
+        g_mutex_lock (&priv->mutex);
+#else
         g_mutex_lock (priv->mutex);
+#endif
         stack_fix_size (priv);
+#if GLIB_CHECK_VERSION(2, 32, 0)
+        g_mutex_unlock (&priv->mutex);
+#else
         g_mutex_unlock (priv->mutex);
+#endif
         do_menu_update (manager);
       }
       break;
@@ -411,7 +436,12 @@ caja_undostack_manager_redo (CajaUndoStackManager * manager,
   char *puri;
   CajaUndoStackManagerPrivate *priv = manager->priv;
 
+#if GLIB_CHECK_VERSION(2, 32, 0)
+  g_mutex_lock (&priv->mutex);
+#else
   g_mutex_lock (priv->mutex);
+#endif
+
 
   CajaUndoStackActionData *action = stack_scroll_left (priv);
 
@@ -420,7 +450,12 @@ caja_undostack_manager_redo (CajaUndoStackManager * manager,
     action->locked = TRUE;
   }
 
+#if GLIB_CHECK_VERSION(2, 32, 0)
+  g_mutex_unlock (&priv->mutex);
+#else
   g_mutex_unlock (priv->mutex);
+#endif
+
 
   do_menu_update (manager);
 
@@ -432,7 +467,7 @@ caja_undostack_manager_redo (CajaUndoStackManager * manager,
         uris = construct_gfile_list (action->sources, action->src_dir);
         caja_file_operations_copy (uris, NULL,
             action->dest_dir, NULL, undo_redo_done_transfer_callback, action);
-        eel_g_object_list_free (uris);
+    	g_list_free_full (uris, g_object_unref);
         break;
       case CAJA_UNDOSTACK_CREATEFILEFROMTEMPLATE:
         puri = get_uri_parent (action->target_uri);
@@ -448,14 +483,14 @@ caja_undostack_manager_redo (CajaUndoStackManager * manager,
         uris = construct_gfile_list (action->sources, action->src_dir);
         caja_file_operations_duplicate (uris, NULL, NULL,
             undo_redo_done_transfer_callback, action);
-        eel_g_object_list_free (uris);
+    	g_list_free_full (uris, g_object_unref);
         break;
       case CAJA_UNDOSTACK_RESTOREFROMTRASH:
       case CAJA_UNDOSTACK_MOVE:
         uris = construct_gfile_list (action->sources, action->src_dir);
         caja_file_operations_move (uris, NULL,
             action->dest_dir, NULL, undo_redo_done_transfer_callback, action);
-        eel_g_object_list_free (uris);
+    	g_list_free_full (uris, g_object_unref);
         break;
       case CAJA_UNDOSTACK_RENAME:
         new_name = get_uri_basename (action->new_uri);
@@ -489,14 +524,14 @@ caja_undostack_manager_redo (CajaUndoStackManager * manager,
           caja_file_operations_trash_or_delete
               (uris, NULL, undo_redo_done_delete_callback, action);
           g_list_free (uri_to_trash);
-          eel_g_object_list_free (uris);
+    	  g_list_free_full (uris, g_object_unref);
         }
         break;
       case CAJA_UNDOSTACK_CREATELINK:
         uris = construct_gfile_list (action->sources, action->src_dir);
         caja_file_operations_link (uris, NULL,
             action->dest_dir, NULL, undo_redo_done_transfer_callback, action);
-        eel_g_object_list_free (uris);
+    	g_list_free_full (uris, g_object_unref);
         break;
       case CAJA_UNDOSTACK_SETPERMISSIONS:
         file = caja_file_get_by_uri (action->target_uri);
@@ -550,7 +585,11 @@ caja_undostack_manager_undo (CajaUndoStackManager * manager,
   char *new_name;
   CajaUndoStackManagerPrivate *priv = manager->priv;
 
+#if GLIB_CHECK_VERSION(2, 32, 0)
+  g_mutex_lock (&priv->mutex);
+#else
   g_mutex_lock (priv->mutex);
+#endif
 
   CajaUndoStackActionData *action = stack_scroll_right (priv);
 
@@ -558,7 +597,11 @@ caja_undostack_manager_undo (CajaUndoStackManager * manager,
     action->locked = TRUE;
   }
 
+#if GLIB_CHECK_VERSION(2, 32, 0)
+  g_mutex_unlock (&priv->mutex);
+#else
   g_mutex_unlock (priv->mutex);
+#endif
 
   do_menu_update (manager);
 
@@ -579,7 +622,7 @@ caja_undostack_manager_undo (CajaUndoStackManager * manager,
         if (priv->confirm_delete) {
           caja_file_operations_delete (uris, NULL,
               undo_redo_done_delete_callback, action);
-          eel_g_object_list_free (uris);
+    	  g_list_free_full (uris, g_object_unref);
         } else {
           /* We skip the confirmation message
            */
@@ -600,7 +643,7 @@ caja_undostack_manager_undo (CajaUndoStackManager * manager,
         uris = construct_gfile_list (action->destinations, action->dest_dir);
         caja_file_operations_trash_or_delete (uris, NULL,
             undo_redo_done_delete_callback, action);
-        eel_g_object_list_free (uris);
+    	g_list_free_full (uris, g_object_unref);
         break;
       case CAJA_UNDOSTACK_MOVETOTRASH:
         files_to_restore = retrieve_files_to_restore (action->trashed);
@@ -630,7 +673,7 @@ caja_undostack_manager_undo (CajaUndoStackManager * manager,
         uris = construct_gfile_list (action->destinations, action->dest_dir);
         caja_file_operations_move (uris, NULL,
             action->src_dir, NULL, undo_redo_done_transfer_callback, action);
-        eel_g_object_list_free (uris);
+    	g_list_free_full (uris, g_object_unref);
         break;
       case CAJA_UNDOSTACK_RENAME:
         new_name = get_uri_basename (action->old_uri);
@@ -714,11 +757,19 @@ caja_undostack_manager_add_action (CajaUndoStackManager * manager,
 
   action->manager = manager;
 
+#if GLIB_CHECK_VERSION(2, 32, 0)
+  g_mutex_lock (&priv->mutex);
+#else
   g_mutex_lock (priv->mutex);
+#endif
 
   stack_push_action (priv, action);
 
+#if GLIB_CHECK_VERSION(2, 32, 0)
+  g_mutex_unlock (&priv->mutex);
+#else
   g_mutex_unlock (priv->mutex);
+#endif
 
   do_menu_update (manager);
 
@@ -774,7 +825,12 @@ caja_undostack_manager_trash_has_emptied (CajaUndoStackManager *
 
   /* Clear actions from the oldest to the newest move to trash */
 
+#if GLIB_CHECK_VERSION(2, 32, 0)
+  g_mutex_lock (&priv->mutex);
+#else
   g_mutex_lock (priv->mutex);
+#endif
+
   clear_redo_actions (priv);
   CajaUndoStackActionData *action = NULL;
 
@@ -799,7 +855,11 @@ caja_undostack_manager_trash_has_emptied (CajaUndoStackManager *
   }
 
   g_queue_free (tmp_stack);
+#if GLIB_CHECK_VERSION(2, 32, 0)
+  g_mutex_unlock (&priv->mutex);
+#else
   g_mutex_unlock (priv->mutex);
+#endif
   do_menu_update (manager);
 }
 
@@ -1759,12 +1819,10 @@ free_undostack_action (gpointer data, gpointer user_data)
   g_free (action->new_user_name_or_id);
 
   if (action->sources) {
-    g_list_foreach (action->sources, (GFunc) g_free, NULL);
-    g_list_free (action->sources);
+    g_list_free_full (action->sources, g_free);
   }
   if (action->destinations) {
-    g_list_foreach (action->destinations, (GFunc) g_free, NULL);
-    g_list_free (action->destinations);
+    g_list_free_full (action->destinations, g_free);
   }
 
   if (action->trashed) {
@@ -1864,7 +1922,11 @@ do_menu_update (CajaUndoStackManager * manager)
   CajaUndoStackManagerPrivate *priv = manager->priv;
   CajaUndoStackMenuData *data = g_slice_new0 (CajaUndoStackMenuData);
 
+#if GLIB_CHECK_VERSION(2, 32, 0)
+  g_mutex_lock (&priv->mutex);
+#else
   g_mutex_lock (priv->mutex);
+#endif
 
   action = get_next_undo_action (priv);
   data->undo_label = get_undo_label (action);
@@ -1875,7 +1937,11 @@ do_menu_update (CajaUndoStackManager * manager)
   data->redo_label = get_redo_label (action);
   data->redo_description = get_redo_description (action);
 
+#if GLIB_CHECK_VERSION(2, 32, 0)
+  g_mutex_unlock (&priv->mutex);
+#else
   g_mutex_unlock (priv->mutex);
+#endif
 
   /* Update menus */
   g_signal_emit_by_name (manager, "request-menu-update", data);
