@@ -18,7 +18,7 @@
  *
  *  You should have received a copy of the GNU General Public License
  *  along with this program; if not, write to the Free Software
- *  Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
+ *  Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1301, USA.
  *
  *  Authors: Elliot Lee <sopwith@redhat.com>,
  *           Darin Adler <darin@bentspoon.com>
@@ -55,6 +55,7 @@
 #include "libcaja-private/caja-file-operations.h"
 #include "caja-window-private.h"
 #include "caja-window-manage-views.h"
+#include "caja-freedesktop-dbus.h"
 #include <unistd.h>
 #include <libxml/xmlsave.h>
 #include <glib/gstdio.h>
@@ -69,12 +70,15 @@
 #include <libcaja-private/caja-file-utilities.h>
 #include <libcaja-private/caja-global-preferences.h>
 #include <libcaja-private/caja-module.h>
-#include <libcaja-private/caja-undo-manager.h>
 #include <libcaja-private/caja-desktop-link-monitor.h>
 #include <libcaja-private/caja-directory-private.h>
 #include <libcaja-private/caja-signaller.h>
 #include <libcaja-extension/caja-menu-provider.h>
 #include <libcaja-private/caja-autorun.h>
+#define MATE_DESKTOP_USE_UNSTABLE_API
+#include <libmateui/mate-bg.h>
+
+#include "glibcompat.h" /* for g_list_free_full */
 
 enum {
 	COMMAND_0, /* unused: 0 is an invalid command */
@@ -104,6 +108,9 @@ static GList *caja_application_spatial_window_list;
 /* The saving of the accelerator map was requested  */
 static gboolean save_of_accel_map_requested = FALSE;
 
+/* File Manager DBus Interface */
+static CajaFreedesktopDBus *fdb_manager = NULL;
+
 static void     desktop_changed_callback          (gpointer                  user_data);
 static void     desktop_location_changed_callback (gpointer                  user_data);
 static void     mount_removed_callback            (GVolumeMonitor            *monitor,
@@ -234,7 +241,7 @@ automount_all_volumes (CajaApplication *application)
     GMount *mount;
     GVolume *volume;
 
-    if (eel_preferences_get_boolean (CAJA_PREFERENCES_MEDIA_AUTOMOUNT))
+    if (g_settings_get_boolean (caja_media_preferences, CAJA_PREFERENCES_MEDIA_AUTOMOUNT))
     {
         /* automount all mountable volumes at start-up */
         volumes = g_volume_monitor_get_volumes (application->volume_monitor);
@@ -258,7 +265,7 @@ automount_all_volumes (CajaApplication *application)
             /* pass NULL as GMountOperation to avoid user interaction */
             g_volume_mount (volume, 0, NULL, NULL, startup_volume_mount_cb, NULL);
         }
-        eel_g_object_list_free (volumes);
+    	g_list_free_full (volumes, g_object_unref);
     }
 
 }
@@ -291,9 +298,6 @@ smclient_quit_cb (EggSMClient   *client,
 static void
 caja_application_init (CajaApplication *application)
 {
-    /* Create an undo manager */
-    application->undo_manager = caja_undo_manager_new ();
-
     application->unique_app = unique_app_new_with_commands ("org.mate.Caja", NULL,
                               "start_desktop", COMMAND_START_DESKTOP,
                               "stop_desktop", COMMAND_STOP_DESKTOP,
@@ -350,8 +354,6 @@ caja_application_finalize (GObject *object)
 
     caja_bookmarks_exiting ();
 
-    g_object_unref (application->undo_manager);
-
     if (application->volume_monitor)
     {
         g_object_unref (application->volume_monitor);
@@ -372,6 +374,12 @@ caja_application_finalize (GObject *object)
         application->proxy = NULL;
     }
 
+    if (fdb_manager != NULL)
+    {
+        g_object_unref (fdb_manager);
+        fdb_manager = NULL;
+    }
+
     G_OBJECT_CLASS (caja_application_parent_class)->finalize (object);
 }
 
@@ -438,7 +446,11 @@ check_required_directories (CajaApplication *application)
 
         dialog = eel_show_error_dialog (error_string, detail_string, NULL);
         /* We need the main event loop so the user has a chance to see the dialog. */
+#if GTK_CHECK_VERSION (3, 0, 0)
+        caja_main_event_loop_register (GTK_WIDGET (dialog));
+#else
         caja_main_event_loop_register (GTK_OBJECT (dialog));
+#endif
 
         g_string_free (directories_as_string, TRUE);
         g_free (error_string);
@@ -462,12 +474,10 @@ menu_provider_items_updated_handler (CajaMenuProvider *provider, GtkWidget* pare
 static void
 menu_provider_init_callback (void)
 {
-    GList *items;
     GList *providers;
     GList *l;
 
     providers = caja_module_get_extensions_for_type (CAJA_TYPE_MENU_PROVIDER);
-    items = NULL;
 
     for (l = providers; l != NULL; l = l->next)
     {
@@ -766,8 +776,7 @@ finish_startup (CajaApplication *application,
     /* listen for eject button presses */
     drives = g_volume_monitor_get_connected_drives (application->volume_monitor);
     g_list_foreach (drives, (GFunc) drive_listen_for_eject_button, application);
-    g_list_foreach (drives, (GFunc) g_object_unref, NULL);
-    g_list_free (drives);
+    g_list_free_full (drives, g_object_unref);
 
     application->automount_idle_id =
         g_idle_add_full (G_PRIORITY_LOW,
@@ -784,42 +793,30 @@ open_window (CajaApplication *application,
     GFile *location;
     CajaWindow *window;
 
+    if (uri == NULL) {
+    	location = g_file_new_for_path (g_get_home_dir ());
+    } else {
+    	location = g_file_new_for_uri (uri);
+    }
+
     if (browser_window ||
-            eel_preferences_get_boolean (CAJA_PREFERENCES_ALWAYS_USE_BROWSER))
-    {
+            g_settings_get_boolean (caja_preferences, CAJA_PREFERENCES_ALWAYS_USE_BROWSER)) {
         window = caja_application_create_navigation_window (application,
                  startup_id,
                  screen);
-        if (uri == NULL)
-        {
-            caja_window_go_home (window);
-        }
-        else
-        {
-            location = g_file_new_for_uri (uri);
-            caja_window_go_to (window, location);
-            g_object_unref (location);
-        }
-    }
-    else
-    {
-        if (uri == NULL)
-        {
-            location = g_file_new_for_path (g_get_home_dir ());
-        }
-        else
-        {
-            location = g_file_new_for_uri (uri);
-        }
-
-        window = caja_application_present_spatial_window (application,
+    } else {
+        window = caja_application_get_spatial_window (application,
                  NULL,
                  startup_id,
                  location,
-                 screen);
-        g_object_unref (location);
+                 screen,
+                 NULL);
     }
 
+    caja_window_go_to (window, location);
+
+    g_object_unref (location);
+
     if (geometry != NULL && !gtk_widget_get_visible (GTK_WIDGET (window)))
     {
         /* never maximize windows opened from shell if a
@@ -859,6 +856,29 @@ open_windows (CajaApplication *application,
     }
 }
 
+void
+caja_application_open_location (CajaApplication *application,
+                                GFile *location,
+                                GFile *selection,
+                                const char *startup_id)
+{
+    CajaWindow *window;
+    GList *sel_list = NULL;
+
+    window = caja_application_create_navigation_window (application, startup_id, gdk_screen_get_default ());
+
+    if (selection != NULL) {
+        sel_list = g_list_prepend (NULL, g_object_ref (selection));
+    }
+
+    caja_window_slot_open_location_full (caja_window_get_active_slot (window), location,
+                                         0, CAJA_WINDOW_OPEN_FLAG_NEW_WINDOW, sel_list, NULL, NULL);
+
+    if (sel_list != NULL) {
+        caja_file_list_free (sel_list);
+    }
+}
+
 static UniqueResponse
 message_received_cb (UniqueApp         *unique_app,
                      gint               command,
@@ -974,7 +994,7 @@ caja_application_startup (CajaApplication *application,
         char *accel_map_filename;
 
         if (!no_desktop &&
-                !eel_preferences_get_boolean (CAJA_PREFERENCES_SHOW_DESKTOP))
+            !g_settings_get_boolean (mate_background_preferences, MATE_BG_KEY_SHOW_DESKTOP))
         {
             no_desktop = TRUE;
         }
@@ -998,19 +1018,20 @@ caja_application_startup (CajaApplication *application,
             g_signal_connect (application->unique_app, "message-received", G_CALLBACK (message_received_cb), application);
         }
 
+        /* Start the File Manager DBus Interface */
+        fdb_manager = caja_freedesktop_dbus_new (application);
+
         /* Monitor the preference to show or hide the desktop */
-        eel_preferences_add_callback_while_alive (CAJA_PREFERENCES_SHOW_DESKTOP,
-                desktop_changed_callback,
-                application,
-                G_OBJECT (application));
+        g_signal_connect_swapped (mate_background_preferences,
+                                  "changed::" MATE_BG_KEY_SHOW_DESKTOP,
+                                  G_CALLBACK(desktop_changed_callback),
+                                  G_OBJECT (application));
 
         /* Monitor the preference to have the desktop */
         /* point to the Unix home folder */
-        eel_preferences_add_callback_while_alive (CAJA_PREFERENCES_DESKTOP_IS_HOME_DIR,
-                desktop_location_changed_callback,
-                NULL,
-                G_OBJECT (application));
-
+        g_signal_connect_swapped (caja_preferences, "changed::" CAJA_PREFERENCES_DESKTOP_IS_HOME_DIR,
+                                  G_CALLBACK(desktop_location_changed_callback),
+                                  G_OBJECT (application));
         /* Create the other windows. */
         if (urls != NULL || !no_default_window)
         {
@@ -1188,9 +1209,7 @@ caja_application_close_desktop (void)
 {
     if (caja_application_desktop_windows != NULL)
     {
-        g_list_foreach (caja_application_desktop_windows,
-                        (GFunc) gtk_widget_destroy, NULL);
-        g_list_free (caja_application_desktop_windows);
+        g_list_free_full (caja_application_desktop_windows, (GDestroyNotify) gtk_widget_destroy);
         caja_application_desktop_windows = NULL;
     }
 }
@@ -1234,22 +1253,25 @@ caja_application_get_existing_spatial_window (GFile *location)
 {
     GList *l;
     CajaWindowSlot *slot;
+    GFile *window_location;
 
     for (l = caja_application_get_spatial_window_list ();
-            l != NULL; l = l->next)
-    {
-        GFile *window_location;
-
+            l != NULL; l = l->next) {
         slot = CAJA_WINDOW (l->data)->details->active_pane->active_slot;
-        window_location = slot->location;
-        if (window_location != NULL)
-        {
-            if (g_file_equal (location, window_location))
-            {
-                return CAJA_SPATIAL_WINDOW (l->data);
+
+        window_location = slot->pending_location;
+
+        if (window_location == NULL) {
+        	window_location = slot->location;
+        }
+
+        if (window_location != NULL) {
+        	if (g_file_equal (location, window_location)) {
+            	return CAJA_SPATIAL_WINDOW (l->data);
             }
         }
     }
+
     return NULL;
 }
 
@@ -1361,7 +1383,11 @@ caja_application_close_all_spatial_windows (void)
 }
 
 static void
+#if GTK_CHECK_VERSION (3, 0, 0)
+caja_application_destroyed_window (GtkWidget *object, CajaApplication *application)
+#else
 caja_application_destroyed_window (GtkObject *object, CajaApplication *application)
+#endif
 {
     caja_application_window_list = g_list_remove (caja_application_window_list, object);
 }
@@ -1425,68 +1451,32 @@ spatial_window_destroyed_callback (void *user_data, GObject *window)
 }
 
 CajaWindow *
-caja_application_present_spatial_window (CajaApplication *application,
-        CajaWindow      *requesting_window,
-        const char          *startup_id,
-        GFile               *location,
-        GdkScreen           *screen)
-{
-    return caja_application_present_spatial_window_with_selection (application,
-            requesting_window,
-            startup_id,
-            location,
-            NULL,
-            screen);
-}
-
-CajaWindow *
-caja_application_present_spatial_window_with_selection (CajaApplication *application,
-        CajaWindow      *requesting_window,
-        const char          *startup_id,
-        GFile               *location,
-        GList		*new_selection,
-        GdkScreen           *screen)
+caja_application_get_spatial_window (CajaApplication *application,
+                                    CajaWindow      *requesting_window,
+                                    const char      *startup_id,
+                                    GFile           *location,
+                                    GdkScreen       *screen,
+                                    gboolean        *existing)
 {
     CajaWindow *window;
-    GList *l;
-    char *uri;
+    gchar *uri;
 
     g_return_val_if_fail (CAJA_IS_APPLICATION (application), NULL);
+    window = CAJA_WINDOW
+    		(caja_application_get_existing_spatial_window (location));
 
-    for (l = caja_application_get_spatial_window_list ();
-            l != NULL; l = l->next)
-    {
-        CajaWindow *existing_window;
-        CajaWindowSlot *slot;
-        GFile *existing_location;
-
-        existing_window = CAJA_WINDOW (l->data);
-        slot = existing_window->details->active_pane->active_slot;
-        existing_location = slot->pending_location;
-
-        if (existing_location == NULL)
-        {
-            existing_location = slot->location;
+	if (window != NULL) {
+		if (existing != NULL) {
+			*existing = TRUE;
         }
 
-        if (g_file_equal (existing_location, location))
-        {
-            gtk_window_present (GTK_WINDOW (existing_window));
-            if (new_selection &&
-                    slot->content_view != NULL)
-            {
-                caja_view_set_selection (slot->content_view, new_selection);
-            }
-
-            uri = g_file_get_uri (location);
-            caja_debug_log (FALSE, CAJA_DEBUG_LOG_DOMAIN_USER,
-                            "present EXISTING spatial window=%p: %s",
-                            existing_window, uri);
-            g_free (uri);
-            return existing_window;
-        }
+		return window;
     }
 
+	if (existing != NULL) {
+		*existing = FALSE;
+	}
+
     window = create_window (application, CAJA_TYPE_SPATIAL_WINDOW, startup_id, screen);
     if (requesting_window)
     {
@@ -1517,8 +1507,6 @@ caja_application_present_spatial_window_with_selection (CajaApplication *applica
     g_object_weak_ref (G_OBJECT (window),
                        spatial_window_destroyed_callback, NULL);
 
-    caja_window_go_to_with_selection (window, location, new_selection);
-
     uri = g_file_get_uri (location);
     caja_debug_log (FALSE, CAJA_DEBUG_LOG_DOMAIN_USER,
                     "present NEW spatial window=%p: %s",
@@ -1559,8 +1547,8 @@ caja_application_create_navigation_window (CajaApplication *application,
 
     window = create_window (application, CAJA_TYPE_NAVIGATION_WINDOW, startup_id, screen);
 
-    maximized = eel_preferences_get_boolean
-                (CAJA_PREFERENCES_NAVIGATION_WINDOW_MAXIMIZED);
+    maximized = g_settings_get_boolean (caja_window_state,
+                    CAJA_WINDOW_STATE_MAXIMIZED);
     if (maximized)
     {
         gtk_window_maximize (GTK_WINDOW (window));
@@ -1570,8 +1558,8 @@ caja_application_create_navigation_window (CajaApplication *application,
         gtk_window_unmaximize (GTK_WINDOW (window));
     }
 
-    geometry_string = eel_preferences_get
-                      (CAJA_PREFERENCES_NAVIGATION_WINDOW_SAVED_GEOMETRY);
+    geometry_string = g_settings_get_string (caja_window_state,
+                        CAJA_WINDOW_STATE_GEOMETRY);
     if (geometry_string != NULL &&
             geometry_string[0] != 0)
     {
@@ -1613,7 +1601,7 @@ desktop_changed_callback (gpointer user_data)
     CajaApplication *application;
 
     application = CAJA_APPLICATION (user_data);
-    if ( eel_preferences_get_boolean (CAJA_PREFERENCES_SHOW_DESKTOP))
+    if (g_settings_get_boolean (mate_background_preferences, MATE_BG_KEY_SHOW_DESKTOP))
     {
         caja_application_open_desktop (application);
     }
@@ -1639,7 +1627,7 @@ volume_added_callback (GVolumeMonitor *monitor,
                        GVolume *volume,
                        CajaApplication *application)
 {
-    if (eel_preferences_get_boolean (CAJA_PREFERENCES_MEDIA_AUTOMOUNT) &&
+    if (g_settings_get_boolean (caja_media_preferences, CAJA_PREFERENCES_MEDIA_AUTOMOUNT) &&
             g_volume_should_automount (volume) &&
             g_volume_can_mount (volume))
     {
@@ -1713,27 +1701,30 @@ autorun_show_window (GMount *mount, gpointer user_data)
 {
     GFile *location;
     CajaApplication *application = user_data;
+    CajaWindow *window;
+    gboolean existing;
 
     location = g_mount_get_root (mount);
+    existing = FALSE;
 
-    /* Ther should probably be an easier way to do this */
-    if (eel_preferences_get_boolean (CAJA_PREFERENCES_ALWAYS_USE_BROWSER))
-    {
-        CajaWindow *window;
+    /* There should probably be an easier way to do this */
+    if (g_settings_get_boolean (caja_preferences, CAJA_PREFERENCES_ALWAYS_USE_BROWSER)) {
         window = caja_application_create_navigation_window (application,
-                 NULL,
-                 gdk_screen_get_default ());
-        caja_window_go_to (window, location);
-
+                                                            NULL,
+                                                            gdk_screen_get_default ());
     }
     else
     {
-        caja_application_present_spatial_window (application,
-                NULL,
-                NULL,
-                location,
-                gdk_screen_get_default ());
+        window = caja_application_get_spatial_window (application,
+                                                      NULL,
+                                                      NULL,
+                                                      location,
+                                                      gdk_screen_get_default (),
+                                                      NULL);
     }
+
+    caja_window_go_to (window, location);
+
     g_object_unref (location);
 }
 
@@ -2294,7 +2285,12 @@ caja_application_load_session (CajaApplication *application)
                 else if (!strcmp (type, "spatial"))
                 {
                     location = g_file_new_for_uri (location_uri);
-                    window = caja_application_present_spatial_window (application, NULL, NULL, location, gdk_screen_get_default ());
+                    window = caja_application_get_spatial_window (application, NULL, NULL, 
+                    											  location, gdk_screen_get_default (),
+                    											  NULL);
+
+					caja_window_go_to (window, location);
+
                     g_object_unref (location);
                 }
                 else
