@@ -17,7 +17,7 @@
  *
  * You should have received a copy of the GNU General Public License
  * along with this program; if not, write to the Free Software
- * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
+ * Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1301, USA.
  *
  * Authors: John Sullivan <sullivan@eazel.com>
  */
@@ -30,11 +30,9 @@
 #include "caja-window.h"
 #include "caja-navigation-window.h"
 #include "caja-spatial-window.h"
-#include <libcaja-private/caja-undo.h>
 #include <libcaja-private/caja-global-preferences.h>
 #include <eel/eel-gtk-extensions.h>
 #include <eel/eel-mate-extensions.h>
-#include <libcaja-private/caja-undo-signal-handlers.h>
 #include <gtk/gtk.h>
 #include <gdk/gdkkeysyms.h>
 
@@ -231,9 +229,9 @@ static void
 edit_bookmarks_dialog_reset_signals (gpointer data,
                                      GObject *obj)
 {
-    g_signal_handler_disconnect (GTK_OBJECT (jump_button),
+    g_signal_handler_disconnect (jump_button,
                                  jump_button_signal_id);
-    g_signal_handler_disconnect (GTK_OBJECT (bookmark_list_widget),
+    g_signal_handler_disconnect (bookmark_list_widget,
                                  row_activated_signal_id);
     jump_button_signal_id =
         g_signal_connect (jump_button, "clicked",
@@ -252,7 +250,7 @@ edit_bookmarks_dialog_reset_signals (gpointer data,
  * Return value: A pointer to the new window.
  **/
 GtkWindow *
-create_bookmarks_window (CajaBookmarkList *list, GObject *undo_manager_source)
+create_bookmarks_window (CajaBookmarkList *list, CajaWindow *window_source)
 {
     GtkWidget         *window;
     GtkTreeViewColumn *col;
@@ -274,9 +272,9 @@ create_bookmarks_window (CajaBookmarkList *list, GObject *undo_manager_source)
     remove_button = (GtkWidget *)gtk_builder_get_object (builder, "bookmark_delete_button");
     jump_button = (GtkWidget *)gtk_builder_get_object (builder, "bookmark_jump_button");
 
-    application = CAJA_WINDOW (undo_manager_source)->application;
+    application = window_source->application;
 
-    if (CAJA_IS_NAVIGATION_WINDOW (undo_manager_source))
+    if (CAJA_IS_NAVIGATION_WINDOW (window_source))
     {
         parent_is_browser_window = TRUE;
     }
@@ -286,13 +284,12 @@ create_bookmarks_window (CajaBookmarkList *list, GObject *undo_manager_source)
     }
 
     set_up_close_accelerator (window);
-    caja_undo_share_undo_manager (G_OBJECT (window), undo_manager_source);
 
     gtk_window_set_wmclass (GTK_WINDOW (window), "bookmarks", "Caja");
     caja_bookmarks_window_restore_geometry (window);
 
-    g_object_weak_ref (G_OBJECT (undo_manager_source), edit_bookmarks_dialog_reset_signals,
-                       undo_manager_source);
+    g_object_weak_ref (G_OBJECT (window_source), edit_bookmarks_dialog_reset_signals,
+                       window_source);
 
     bookmark_list_widget = GTK_TREE_VIEW (gtk_builder_get_object (builder, "bookmark_tree_view"));
 
@@ -336,7 +333,6 @@ create_bookmarks_window (CajaBookmarkList *list, GObject *undo_manager_source)
     gtk_widget_show (name_field);
     gtk_box_pack_start (GTK_BOX (gtk_builder_get_object (builder, "bookmark_name_placeholder")),
                         name_field, TRUE, TRUE, 0);
-    caja_undo_editable_set_undo_key (GTK_EDITABLE (name_field), TRUE);
 
     gtk_label_set_mnemonic_widget (
         GTK_LABEL (gtk_builder_get_object (builder, "bookmark_name_label")),
@@ -346,7 +342,6 @@ create_bookmarks_window (CajaBookmarkList *list, GObject *undo_manager_source)
     gtk_widget_show (uri_field);
     gtk_box_pack_start (GTK_BOX (gtk_builder_get_object (builder, "bookmark_location_placeholder")),
                         uri_field, TRUE, TRUE, 0);
-    caja_undo_editable_set_undo_key (GTK_EDITABLE (uri_field), TRUE);
 
     gtk_label_set_mnemonic_widget (
         GTK_LABEL (gtk_builder_get_object (builder, "bookmark_location_label")),
@@ -363,7 +358,7 @@ create_bookmarks_window (CajaBookmarkList *list, GObject *undo_manager_source)
                           G_CALLBACK (on_row_deleted), NULL);
     row_activated_signal_id =
         g_signal_connect (bookmark_list_widget, "row_activated",
-                          G_CALLBACK (on_row_activated), undo_manager_source);
+                          G_CALLBACK (on_row_activated), window_source);
     button_pressed_signal_id =
         g_signal_connect (bookmark_list_widget, "button_press_event",
                           G_CALLBACK (on_button_pressed), NULL);
@@ -404,7 +399,7 @@ create_bookmarks_window (CajaBookmarkList *list, GObject *undo_manager_source)
                       G_CALLBACK (on_remove_button_clicked), NULL);
     jump_button_signal_id =
         g_signal_connect (jump_button, "clicked",
-                          G_CALLBACK (on_jump_button_clicked), undo_manager_source);
+                          G_CALLBACK (on_jump_button_clicked), window_source);
 
     gtk_tree_selection_set_mode (bookmark_selection, GTK_SELECTION_BROWSE);
 
@@ -417,23 +412,23 @@ create_bookmarks_window (CajaBookmarkList *list, GObject *undo_manager_source)
 }
 
 void
-edit_bookmarks_dialog_set_signals (GObject *undo_manager_source)
+edit_bookmarks_dialog_set_signals (CajaWindow *window)
 {
 
-    g_signal_handler_disconnect (GTK_OBJECT (jump_button),
+    g_signal_handler_disconnect (jump_button,
                                  jump_button_signal_id);
-    g_signal_handler_disconnect (GTK_OBJECT (bookmark_list_widget),
+    g_signal_handler_disconnect (bookmark_list_widget,
                                  row_activated_signal_id);
 
     jump_button_signal_id =
         g_signal_connect (jump_button, "clicked",
-                          G_CALLBACK (on_jump_button_clicked), undo_manager_source);
+                          G_CALLBACK (on_jump_button_clicked), window);
     row_activated_signal_id =
         g_signal_connect (bookmark_list_widget, "row_activated",
-                          G_CALLBACK (on_row_activated), undo_manager_source);
+                          G_CALLBACK (on_row_activated), window);
 
-    g_object_weak_ref (G_OBJECT (undo_manager_source), edit_bookmarks_dialog_reset_signals,
-                       undo_manager_source);
+    g_object_weak_ref (G_OBJECT (window), edit_bookmarks_dialog_reset_signals,
+                       window);
 }
 
 static CajaBookmark *
@@ -587,35 +582,35 @@ open_selected_bookmark (gpointer user_data, GdkScreen *screen)
 
     if (CAJA_IS_NAVIGATION_WINDOW (user_data))
     {
-        caja_window_go_to (CAJA_WINDOW (user_data), location);
+        window = user_data;
     }
     else if (CAJA_IS_SPATIAL_WINDOW (user_data))
     {
-        window = caja_application_present_spatial_window (application,
-                 NULL,
-                 NULL,
-                 location,
-                 screen);
-    }
-    else     /* window that opened bookmarks window has been closed */
-    {
-        if (parent_is_browser_window || eel_preferences_get_boolean (CAJA_PREFERENCES_ALWAYS_USE_BROWSER))
-        {
+        window = caja_application_get_spatial_window (application,
+                                                      NULL,
+                                                      NULL,
+                                                      location,
+                                                      screen,
+                                                      NULL);
+    } else { /* window that opened bookmarks window has been closed */
+        if (parent_is_browser_window || g_settings_get_boolean (caja_preferences, CAJA_PREFERENCES_ALWAYS_USE_BROWSER)) {
             window = caja_application_create_navigation_window (application,
                      NULL,
                      screen);
-            caja_window_go_to (window, location);
         }
         else
         {
-            window = caja_application_present_spatial_window (application,
-                     NULL,
-                     NULL,
-                     location,
-                     screen);
+            window = caja_application_get_spatial_window (application,
+                                                          NULL,
+                                                          NULL,
+                                                          location,
+                                                          screen,
+                                                          NULL);
         }
     }
 
+    caja_window_go_to (window, location);
+
     g_object_unref (location);
 }
 
@@ -754,7 +749,7 @@ on_key_pressed (GtkTreeView *view,
                 GdkEventKey *event,
                 gpointer user_data)
 {
-    if (event->keyval == GDK_Delete || event->keyval == GDK_KP_Delete)
+    if (event->keyval == GDK_KEY_Delete || event->keyval == GDK_KEY_KP_Delete)
     {
         bookmarks_delete_bookmark ();
         return TRUE;
@@ -962,10 +957,6 @@ on_window_hide_event (GtkWidget *widget,
 {
     caja_bookmarks_window_save_geometry (GTK_WINDOW (widget));
 
-    /* Disable undo for entry widgets */
-    caja_undo_unregister (G_OBJECT (name_field));
-    caja_undo_unregister (G_OBJECT (uri_field));
-
     /* restore_geometry only works after window is hidden */
     g_idle_add (restore_geometry, widget);
 }
@@ -1091,8 +1082,7 @@ handle_close_accelerator (GtkWindow *window,
     g_assert (event != NULL);
     g_assert (user_data == NULL);
 
-    if (eel_gtk_window_event_is_close_accelerator (window, event))
-    {
+	if (event->state & GDK_CONTROL_MASK && event->keyval == GDK_KEY_w) {
         gtk_widget_hide (GTK_WIDGET (window));
         return TRUE;
     }
