@@ -16,8 +16,8 @@
  *
  * You should have received a copy of the GNU General Public
  * License along with this program; if not, write to the
- * Free Software Foundation, Inc., 59 Temple Place - Suite 330,
- * Boston, MA 02111-1307, USA.
+ * Free Software Foundation, Inc., 51 Franklin St, Fifth Floor,
+ * Boston, MA 02110-1301, USA.
  *
  * Author: Alexander Larsson <alexl@redhat.com>
  * XMP support by Hubert Figuiere <hfiguiere@novell.com>
@@ -391,6 +391,10 @@ load_finished (CajaImagePropertiesPage *page)
 
     gtk_widget_destroy (page->details->loading_label);
 
+    if (page->details->loader != NULL) {
+        gdk_pixbuf_loader_close (page->details->loader, NULL);
+    }
+
     if (page->details->got_size)
     {
 #ifdef HAVE_EXIF
@@ -437,7 +441,6 @@ load_finished (CajaImagePropertiesPage *page)
 
     if (page->details->loader != NULL)
     {
-        gdk_pixbuf_loader_close (page->details->loader, NULL);
         g_object_unref (page->details->loader);
         page->details->loader = NULL;
     }
