@@ -16,8 +16,8 @@
 
    You should have received a copy of the GNU General Public
    License along with this program; if not, write to the
-   Free Software Foundation, Inc., 59 Temple Place - Suite 330,
-   Boston, MA 02111-1307, USA.
+   Free Software Foundation, Inc., 51 Franklin St, Fifth Floor,
+   Boston, MA 02110-1301, USA.
 
    Author: Holger Berndt <berndth@gmx.de>
 */
@@ -35,8 +35,6 @@
 #include <libcaja-private/caja-view-factory.h>
 #include <libcaja-private/caja-entry.h>
 
-#include <eel/eel-preferences.h>
-
 
 static void caja_navigation_window_pane_init       (CajaNavigationWindowPane *pane);
 static void caja_navigation_window_pane_class_init (CajaNavigationWindowPaneClass *class);
@@ -219,7 +217,7 @@ path_bar_location_changed_callback (GtkWidget *widget,
 static gboolean
 location_button_should_be_active (CajaNavigationWindowPane *pane)
 {
-    return eel_preferences_get_boolean (CAJA_PREFERENCES_ALWAYS_USE_LOCATION_ENTRY);
+    return g_settings_get_boolean (caja_preferences, CAJA_PREFERENCES_ALWAYS_USE_LOCATION_ENTRY);
 }
 
 static void
@@ -229,7 +227,7 @@ location_button_toggled_cb (GtkToggleButton *toggle,
     gboolean is_active;
 
     is_active = gtk_toggle_button_get_active (toggle);
-    eel_preferences_set_boolean (CAJA_PREFERENCES_ALWAYS_USE_LOCATION_ENTRY, is_active);
+    g_settings_set_boolean (caja_preferences, CAJA_PREFERENCES_ALWAYS_USE_LOCATION_ENTRY, is_active);
 
     if (is_active) {
         caja_navigation_bar_activate (CAJA_NAVIGATION_BAR (pane->navigation_bar));
@@ -354,6 +352,16 @@ path_bar_button_drag_begin_callback (GtkWidget *widget,
                        GINT_TO_POINTER (FALSE));
 }
 
+static void
+notebook_popup_menu_new_tab_cb (GtkMenuItem *menuitem,
+    			gpointer user_data)
+{
+    CajaWindowPane *pane;
+
+    pane = CAJA_WINDOW_PANE (user_data);
+    caja_window_new_tab (pane->window);
+}
+
 static void
 path_bar_path_set_callback (GtkWidget *widget,
                             GFile *location,
@@ -444,6 +452,16 @@ notebook_popup_menu_show (CajaNavigationWindowPane *pane,
 
     popup = gtk_menu_new();
 
+    item = gtk_menu_item_new_with_mnemonic (_("_New Tab"));
+    g_signal_connect (item, "activate",
+    		  G_CALLBACK (notebook_popup_menu_new_tab_cb),
+    		  pane);
+    gtk_menu_shell_append (GTK_MENU_SHELL (popup),
+    		       item);
+
+    gtk_menu_shell_append (GTK_MENU_SHELL (popup),
+    		       gtk_separator_menu_item_new ());
+
     item = gtk_menu_item_new_with_mnemonic (_("Move Tab _Left"));
     g_signal_connect (item, "activate",
                       G_CALLBACK (notebook_popup_menu_move_left_cb),
@@ -653,7 +671,7 @@ caja_navigation_window_pane_hide_temporary_bars (CajaNavigationWindowPane *pane)
         }
         else
         {
-            if (!eel_preferences_get_boolean (CAJA_PREFERENCES_ALWAYS_USE_LOCATION_ENTRY))
+            if (!g_settings_get_boolean (caja_preferences, CAJA_PREFERENCES_ALWAYS_USE_LOCATION_ENTRY))
             {
                 caja_navigation_window_pane_set_bar_mode (pane, CAJA_BAR_PATH);
             }
@@ -667,7 +685,7 @@ caja_navigation_window_pane_hide_temporary_bars (CajaNavigationWindowPane *pane)
     {
         CajaNavigationWindow *window;
 
-        if (!eel_preferences_get_boolean (CAJA_PREFERENCES_ALWAYS_USE_LOCATION_ENTRY))
+        if (!g_settings_get_boolean (caja_preferences, CAJA_PREFERENCES_ALWAYS_USE_LOCATION_ENTRY))
         {
             caja_navigation_window_pane_set_bar_mode (pane, CAJA_BAR_PATH);
         }
@@ -910,9 +928,9 @@ caja_navigation_window_pane_hide_location_bar (CajaNavigationWindowPane *pane, g
     gtk_widget_hide(pane->location_bar);
     caja_navigation_window_update_show_hide_menu_items(
         CAJA_NAVIGATION_WINDOW (CAJA_WINDOW_PANE (pane)->window));
-    if (save_preference && eel_preferences_key_is_writable(CAJA_PREFERENCES_START_WITH_LOCATION_BAR))
+    if (save_preference)
     {
-        eel_preferences_set_boolean(CAJA_PREFERENCES_START_WITH_LOCATION_BAR, FALSE);
+        g_settings_set_boolean (caja_window_state, CAJA_WINDOW_STATE_START_WITH_LOCATION_BAR, FALSE);
     }
 }
 
@@ -921,9 +939,9 @@ caja_navigation_window_pane_show_location_bar (CajaNavigationWindowPane *pane, g
 {
     gtk_widget_show(pane->location_bar);
     caja_navigation_window_update_show_hide_menu_items(CAJA_NAVIGATION_WINDOW (CAJA_WINDOW_PANE (pane)->window));
-    if (save_preference && eel_preferences_key_is_writable(CAJA_PREFERENCES_START_WITH_LOCATION_BAR))
+    if (save_preference)
     {
-        eel_preferences_set_boolean(CAJA_PREFERENCES_START_WITH_LOCATION_BAR, TRUE);
+        g_settings_set_boolean (caja_window_state, CAJA_WINDOW_STATE_START_WITH_LOCATION_BAR, TRUE);
     }
 }
 
