@@ -19,7 +19,7 @@
  *
  *  You should have received a copy of the GNU General Public
  *  License along with this program; if not, write to the Free
- *  Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
+ *  Software Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1301, USA.
  *
  *  Authors: Elliot Lee <sopwith@redhat.com>
  *  	     John Sullivan <sullivan@eazel.com>
@@ -55,7 +55,6 @@
 #include <libcaja-private/caja-file-utilities.h>
 #include <libcaja-private/caja-file-attributes.h>
 #include <libcaja-private/caja-global-preferences.h>
-#include <libcaja-private/caja-horizontal-splitter.h>
 #include <libcaja-private/caja-icon-info.h>
 #include <libcaja-private/caja-metadata.h>
 #include <libcaja-private/caja-mime-actions.h>
@@ -63,7 +62,6 @@
 #include <libcaja-private/caja-sidebar.h>
 #include <libcaja-private/caja-view-factory.h>
 #include <libcaja-private/caja-clipboard.h>
-#include <libcaja-private/caja-undo.h>
 #include <libcaja-private/caja-module.h>
 #include <libcaja-private/caja-sidebar-provider.h>
 #include <libcaja-private/caja-search-directory.h>
@@ -89,7 +87,6 @@ enum
     ARG_APP
 };
 
-static int side_pane_width_auto_value = 0;
 
 
 /* Forward and back buttons on the mouse */
@@ -143,7 +140,11 @@ caja_navigation_window_init (CajaNavigationWindow *window)
     window->details->header_size_group = gtk_size_group_new (GTK_SIZE_GROUP_VERTICAL);
     gtk_size_group_set_ignore_hidden (window->details->header_size_group, FALSE);
 
-    window->details->content_paned = caja_horizontal_splitter_new ();
+#if GTK_CHECK_VERSION(3, 0, 0)
+    window->details->content_paned = gtk_paned_new (GTK_ORIENTATION_HORIZONTAL);
+#else
+    window->details->content_paned = gtk_hpaned_new ();
+#endif
     gtk_table_attach (GTK_TABLE (CAJA_WINDOW (window)->details->table),
                       window->details->content_paned,
                       /* X direction */                   /* Y direction */
@@ -153,10 +154,15 @@ caja_navigation_window_init (CajaNavigationWindow *window)
     gtk_widget_show (window->details->content_paned);
 
     vbox = gtk_vbox_new (FALSE, 0);
-    caja_horizontal_splitter_pack2 (CAJA_HORIZONTAL_SPLITTER (window->details->content_paned), vbox);
+    gtk_paned_pack2 (GTK_PANED (window->details->content_paned), vbox,
+    		     TRUE, FALSE);
     gtk_widget_show (vbox);
 
+#if GTK_CHECK_VERSION(3, 0, 0)    
+    hpaned = gtk_paned_new (GTK_ORIENTATION_HORIZONTAL);
+#else
     hpaned = gtk_hpaned_new ();
+#endif
     gtk_box_pack_start (GTK_BOX (vbox), hpaned, TRUE, TRUE, 0);
     gtk_widget_show (hpaned);
     window->details->split_view_hpane = hpaned;
@@ -196,13 +202,15 @@ caja_navigation_window_init (CajaNavigationWindow *window)
     caja_navigation_window_allow_back (window, FALSE);
     caja_navigation_window_allow_forward (window, FALSE);
 
-    eel_preferences_add_callback_while_alive (CAJA_PREFERENCES_ALWAYS_USE_LOCATION_ENTRY,
-            always_use_location_entry_changed,
-            window, G_OBJECT (window));
+    g_signal_connect_swapped (caja_preferences,
+                              "changed::" CAJA_PREFERENCES_ALWAYS_USE_LOCATION_ENTRY,
+                              G_CALLBACK(always_use_location_entry_changed),
+                              window);
 
-    eel_preferences_add_callback_while_alive (CAJA_PREFERENCES_ALWAYS_USE_BROWSER,
-            always_use_browser_changed,
-            window, G_OBJECT (window));
+    g_signal_connect_swapped (caja_preferences,
+                              "changed::" CAJA_PREFERENCES_ALWAYS_USE_BROWSER,
+                              G_CALLBACK(always_use_browser_changed),
+                              window);
 }
 
 static void
@@ -214,7 +222,7 @@ always_use_location_entry_changed (gpointer callback_data)
 
     window = CAJA_NAVIGATION_WINDOW (callback_data);
 
-    use_entry = eel_preferences_get_boolean (CAJA_PREFERENCES_ALWAYS_USE_LOCATION_ENTRY);
+    use_entry = g_settings_get_boolean (caja_preferences, CAJA_PREFERENCES_ALWAYS_USE_LOCATION_ENTRY);
 
     for (walk = CAJA_WINDOW(window)->details->panes; walk; walk = walk->next)
     {
@@ -246,7 +254,7 @@ mouse_back_button_changed (gpointer callback_data)
 {
     int new_back_button;
 
-    new_back_button = eel_preferences_get_integer (CAJA_PREFERENCES_MOUSE_BACK_BUTTON);
+    new_back_button = g_settings_get_int (caja_preferences, CAJA_PREFERENCES_MOUSE_BACK_BUTTON);
 
     /* Bounds checking */
     if (new_back_button < 6 || new_back_button > UPPER_MOUSE_LIMIT)
@@ -260,7 +268,7 @@ mouse_forward_button_changed (gpointer callback_data)
 {
     int new_forward_button;
 
-    new_forward_button = eel_preferences_get_integer (CAJA_PREFERENCES_MOUSE_FORWARD_BUTTON);
+    new_forward_button = g_settings_get_int (caja_preferences, CAJA_PREFERENCES_MOUSE_FORWARD_BUTTON);
 
     /* Bounds checking */
     if (new_forward_button < 6 || new_forward_button > UPPER_MOUSE_LIMIT)
@@ -272,7 +280,7 @@ mouse_forward_button_changed (gpointer callback_data)
 static void
 use_extra_mouse_buttons_changed (gpointer callback_data)
 {
-    mouse_extra_buttons = eel_preferences_get_boolean (CAJA_PREFERENCES_MOUSE_USE_EXTRA_BUTTONS);
+    mouse_extra_buttons = g_settings_get_boolean (caja_preferences, CAJA_PREFERENCES_MOUSE_USE_EXTRA_BUTTONS);
 }
 
 void
@@ -382,34 +390,23 @@ side_pane_size_allocate_callback (GtkWidget *widget,
     if (allocation->width != window->details->side_pane_width)
     {
         window->details->side_pane_width = allocation->width;
-        if (eel_preferences_key_is_writable (CAJA_PREFERENCES_SIDEBAR_WIDTH))
-        {
-            eel_preferences_set_integer
-            (CAJA_PREFERENCES_SIDEBAR_WIDTH,
-             allocation->width <= 1 ? 0 : allocation->width);
-        }
+        g_settings_set_int (caja_window_state,
+                            CAJA_WINDOW_STATE_SIDEBAR_WIDTH,
+                            allocation->width <= 1 ? 0 : allocation->width);
     }
 }
 
 static void
 setup_side_pane_width (CajaNavigationWindow *window)
 {
-    static gboolean setup_auto_value= TRUE;
-
     g_return_if_fail (window->sidebar != NULL);
 
-    if (setup_auto_value)
-    {
-        setup_auto_value = FALSE;
-        eel_preferences_add_auto_integer
-        (CAJA_PREFERENCES_SIDEBAR_WIDTH,
-         &side_pane_width_auto_value);
-    }
-
-    window->details->side_pane_width = side_pane_width_auto_value;
+    window->details->side_pane_width =
+        g_settings_get_int (caja_window_state,
+                            CAJA_WINDOW_STATE_SIDEBAR_WIDTH);
 
     gtk_paned_set_position (GTK_PANED (window->details->content_paned),
-                            side_pane_width_auto_value);
+                            window->details->side_pane_width);
 }
 
 static void
@@ -449,10 +446,7 @@ side_pane_switch_page_callback (CajaSidePane *side_pane,
     set_current_side_panel (window, sidebar);
 
     id = caja_sidebar_get_sidebar_id (sidebar);
-    if (eel_preferences_key_is_writable (CAJA_PREFERENCES_SIDE_PANE_VIEW))
-    {
-        eel_preferences_set (CAJA_PREFERENCES_SIDE_PANE_VIEW, id);
-    }
+    g_settings_set_string (caja_window_state, CAJA_WINDOW_STATE_SIDE_PANE_VIEW, id);
 }
 
 static void
@@ -515,24 +509,14 @@ caja_navigation_window_tear_down_sidebar (CajaNavigationWindow *window)
     window->sidebar = NULL;
 }
 
-static void
-caja_navigation_window_unrealize (GtkWidget *widget)
-{
-    CajaNavigationWindow *window;
-
-    window = CAJA_NAVIGATION_WINDOW (widget);
-
-    GTK_WIDGET_CLASS (parent_class)->unrealize (widget);
-}
-
 static gboolean
 caja_navigation_window_state_event (GtkWidget *widget,
                                     GdkEventWindowState *event)
 {
     if (event->changed_mask & GDK_WINDOW_STATE_MAXIMIZED)
     {
-        eel_preferences_set_boolean (CAJA_PREFERENCES_NAVIGATION_WINDOW_MAXIMIZED,
-                                     event->new_window_state & GDK_WINDOW_STATE_MAXIMIZED);
+        g_settings_set_boolean (caja_window_state, CAJA_WINDOW_STATE_MAXIMIZED,
+                                event->new_window_state & GDK_WINDOW_STATE_MAXIMIZED);
     }
 
     if (GTK_WIDGET_CLASS (parent_class)->window_state_event != NULL)
@@ -607,7 +591,11 @@ caja_navigation_window_button_press_event (GtkWidget *widget,
 }
 
 static void
+#if GTK_CHECK_VERSION (3, 0, 0)
+caja_navigation_window_destroy (GtkWidget *object)
+#else
 caja_navigation_window_destroy (GtkObject *object)
+#endif
 {
     CajaNavigationWindow *window;
 
@@ -623,7 +611,11 @@ caja_navigation_window_destroy (GtkObject *object)
     window->details->content_paned = NULL;
     window->details->split_view_hpane = NULL;
 
+#if GTK_CHECK_VERSION (3, 0, 0)
+    GTK_WIDGET_CLASS (parent_class)->destroy (object);
+#else
     GTK_OBJECT_CLASS (parent_class)->destroy (object);
+#endif
 }
 
 static void
@@ -635,6 +627,13 @@ caja_navigation_window_finalize (GObject *object)
 
     caja_navigation_window_remove_go_menu_callback (window);
 
+    g_signal_handlers_disconnect_by_func (caja_preferences,
+                                          always_use_browser_changed,
+                                          window);
+    g_signal_handlers_disconnect_by_func (caja_preferences,
+                                          always_use_location_entry_changed,
+                                          window);
+
     G_OBJECT_CLASS (parent_class)->finalize (object);
 }
 
@@ -683,7 +682,7 @@ caja_navigation_window_add_sidebar_panel (CajaNavigationWindow *window,
 
     /* Show if default */
     sidebar_id = caja_sidebar_get_sidebar_id (sidebar_panel);
-    default_id = eel_preferences_get (CAJA_PREFERENCES_SIDE_PANE_VIEW);
+    default_id = g_settings_get_string (caja_window_state, CAJA_WINDOW_STATE_SIDE_PANE_VIEW);
     if (sidebar_id && default_id && !strcmp (sidebar_id, default_id))
     {
         caja_side_pane_show_panel (window->sidebar,
@@ -750,14 +749,11 @@ static void
 real_sync_title (CajaWindow *window,
                  CajaWindowSlot *slot)
 {
-    CajaNavigationWindow *navigation_window;
     CajaNavigationWindowPane *pane;
     CajaNotebook *notebook;
     char *full_title;
     char *window_title;
 
-    navigation_window = CAJA_NAVIGATION_WINDOW (window);
-
     EEL_CALL_PARENT (CAJA_WINDOW_CLASS,
                      sync_title, (window, slot));
 
@@ -766,7 +762,7 @@ real_sync_title (CajaWindow *window,
         /* if spatial mode is default, we keep "File Browser" in the window title
          * to recognize browser windows. Otherwise, we default to the directory name.
          */
-        if (!eel_preferences_get_boolean (CAJA_PREFERENCES_ALWAYS_USE_BROWSER))
+        if (!g_settings_get_boolean (caja_preferences, CAJA_PREFERENCES_ALWAYS_USE_BROWSER))
         {
             full_title = g_strdup_printf (_("%s - File Browser"), slot->title);
             window_title = eel_str_middle_truncate (full_title, MAX_TITLE_LENGTH);
@@ -960,11 +956,8 @@ caja_navigation_window_hide_status_bar (CajaNavigationWindow *window)
     gtk_widget_hide (CAJA_WINDOW (window)->details->statusbar);
 
     caja_navigation_window_update_show_hide_menu_items (window);
-    if (eel_preferences_key_is_writable (CAJA_PREFERENCES_START_WITH_STATUS_BAR) &&
-            eel_preferences_get_boolean (CAJA_PREFERENCES_START_WITH_STATUS_BAR))
-    {
-        eel_preferences_set_boolean (CAJA_PREFERENCES_START_WITH_STATUS_BAR, FALSE);
-    }
+
+    g_settings_set_boolean (caja_window_state, CAJA_WINDOW_STATE_START_WITH_STATUS_BAR, FALSE);
 }
 
 void
@@ -973,11 +966,8 @@ caja_navigation_window_show_status_bar (CajaNavigationWindow *window)
     gtk_widget_show (CAJA_WINDOW (window)->details->statusbar);
 
     caja_navigation_window_update_show_hide_menu_items (window);
-    if (eel_preferences_key_is_writable (CAJA_PREFERENCES_START_WITH_STATUS_BAR) &&
-            !eel_preferences_get_boolean (CAJA_PREFERENCES_START_WITH_STATUS_BAR))
-    {
-        eel_preferences_set_boolean (CAJA_PREFERENCES_START_WITH_STATUS_BAR, TRUE);
-    }
+
+    g_settings_set_boolean (caja_window_state, CAJA_WINDOW_STATE_START_WITH_STATUS_BAR, TRUE);
 }
 
 gboolean
@@ -997,11 +987,7 @@ caja_navigation_window_hide_toolbar (CajaNavigationWindow *window)
 {
     gtk_widget_hide (window->details->toolbar);
     caja_navigation_window_update_show_hide_menu_items (window);
-    if (eel_preferences_key_is_writable (CAJA_PREFERENCES_START_WITH_TOOLBAR) &&
-            eel_preferences_get_boolean (CAJA_PREFERENCES_START_WITH_TOOLBAR))
-    {
-        eel_preferences_set_boolean (CAJA_PREFERENCES_START_WITH_TOOLBAR, FALSE);
-    }
+    g_settings_set_boolean (caja_window_state, CAJA_WINDOW_STATE_START_WITH_TOOLBAR, FALSE);
 }
 
 void
@@ -1009,11 +995,7 @@ caja_navigation_window_show_toolbar (CajaNavigationWindow *window)
 {
     gtk_widget_show (window->details->toolbar);
     caja_navigation_window_update_show_hide_menu_items (window);
-    if (eel_preferences_key_is_writable (CAJA_PREFERENCES_START_WITH_TOOLBAR) &&
-            !eel_preferences_get_boolean (CAJA_PREFERENCES_START_WITH_TOOLBAR))
-    {
-        eel_preferences_set_boolean (CAJA_PREFERENCES_START_WITH_TOOLBAR, TRUE);
-    }
+    g_settings_set_boolean (caja_window_state, CAJA_WINDOW_STATE_START_WITH_TOOLBAR, TRUE);
 }
 
 void
@@ -1027,11 +1009,7 @@ caja_navigation_window_hide_sidebar (CajaNavigationWindow *window)
     caja_navigation_window_tear_down_sidebar (window);
     caja_navigation_window_update_show_hide_menu_items (window);
 
-    if (eel_preferences_key_is_writable (CAJA_PREFERENCES_START_WITH_SIDEBAR) &&
-            eel_preferences_get_boolean (CAJA_PREFERENCES_START_WITH_SIDEBAR))
-    {
-        eel_preferences_set_boolean (CAJA_PREFERENCES_START_WITH_SIDEBAR, FALSE);
-    }
+    g_settings_set_boolean (caja_window_state, CAJA_WINDOW_STATE_START_WITH_SIDEBAR, FALSE);
 }
 
 void
@@ -1044,11 +1022,7 @@ caja_navigation_window_show_sidebar (CajaNavigationWindow *window)
 
     caja_navigation_window_set_up_sidebar (window);
     caja_navigation_window_update_show_hide_menu_items (window);
-    if (eel_preferences_key_is_writable (CAJA_PREFERENCES_START_WITH_SIDEBAR) &&
-            !eel_preferences_get_boolean (CAJA_PREFERENCES_START_WITH_SIDEBAR))
-    {
-        eel_preferences_set_boolean (CAJA_PREFERENCES_START_WITH_SIDEBAR, TRUE);
-    }
+    g_settings_set_boolean (caja_window_state, CAJA_WINDOW_STATE_START_WITH_SIDEBAR, TRUE);
 }
 
 gboolean
@@ -1057,7 +1031,7 @@ caja_navigation_window_sidebar_showing (CajaNavigationWindow *window)
     g_return_val_if_fail (CAJA_IS_NAVIGATION_WINDOW (window), FALSE);
 
     return (window->sidebar != NULL)
-           && caja_horizontal_splitter_is_hidden (CAJA_HORIZONTAL_SPLITTER (window->details->content_paned));
+           && gtk_widget_get_visible (gtk_paned_get_child1 (GTK_PANED (window->details->content_paned)));
 }
 
 /**
@@ -1112,7 +1086,7 @@ caja_navigation_window_show (GtkWidget *widget)
      * these can be controlled on a per-window basis from View menu items.
      */
 
-    if (eel_preferences_get_boolean (CAJA_PREFERENCES_START_WITH_TOOLBAR))
+    if (g_settings_get_boolean (caja_window_state, CAJA_WINDOW_STATE_START_WITH_TOOLBAR))
     {
         caja_navigation_window_show_toolbar (window);
     }
@@ -1121,8 +1095,8 @@ caja_navigation_window_show (GtkWidget *widget)
         caja_navigation_window_hide_toolbar (window);
     }
 
-    show_location_bar = eel_preferences_get_boolean (CAJA_PREFERENCES_START_WITH_LOCATION_BAR);
-    always_use_location_entry = eel_preferences_get_boolean (CAJA_PREFERENCES_ALWAYS_USE_LOCATION_ENTRY);
+    show_location_bar = g_settings_get_boolean (caja_window_state, CAJA_WINDOW_STATE_START_WITH_LOCATION_BAR);
+    always_use_location_entry = g_settings_get_boolean (caja_preferences, CAJA_PREFERENCES_ALWAYS_USE_LOCATION_ENTRY);
     for (walk = CAJA_WINDOW(window)->details->panes; walk; walk = walk->next)
     {
         CajaNavigationWindowPane *pane = walk->data;
@@ -1145,7 +1119,7 @@ caja_navigation_window_show (GtkWidget *widget)
         }
     }
 
-    if (eel_preferences_get_boolean (CAJA_PREFERENCES_START_WITH_SIDEBAR))
+    if (g_settings_get_boolean (caja_window_state, CAJA_WINDOW_STATE_START_WITH_SIDEBAR))
     {
         caja_navigation_window_show_sidebar (window);
     }
@@ -1154,7 +1128,7 @@ caja_navigation_window_show (GtkWidget *widget)
         caja_navigation_window_hide_sidebar (window);
     }
 
-    if (eel_preferences_get_boolean (CAJA_PREFERENCES_START_WITH_STATUS_BAR))
+    if (g_settings_get_boolean (caja_window_state, CAJA_WINDOW_STATE_START_WITH_STATUS_BAR))
     {
         caja_navigation_window_show_status_bar (window);
     }
@@ -1180,21 +1154,17 @@ caja_navigation_window_save_geometry (CajaNavigationWindow *window)
         is_maximized = gdk_window_get_state (gtk_widget_get_window (GTK_WIDGET (window)))
                        & GDK_WINDOW_STATE_MAXIMIZED;
 
-        if (eel_preferences_key_is_writable (CAJA_PREFERENCES_NAVIGATION_WINDOW_SAVED_GEOMETRY) &&
-                !is_maximized)
+        if (!is_maximized)
         {
-            eel_preferences_set
-            (CAJA_PREFERENCES_NAVIGATION_WINDOW_SAVED_GEOMETRY,
-             geometry_string);
+            g_settings_set_string (caja_window_state,
+                                   CAJA_WINDOW_STATE_GEOMETRY,
+                                   geometry_string);
         }
         g_free (geometry_string);
 
-        if (eel_preferences_key_is_writable (CAJA_PREFERENCES_NAVIGATION_WINDOW_MAXIMIZED))
-        {
-            eel_preferences_set_boolean
-            (CAJA_PREFERENCES_NAVIGATION_WINDOW_MAXIMIZED,
-             is_maximized);
-        }
+        g_settings_set_boolean (caja_window_state,
+                                CAJA_WINDOW_STATE_MAXIMIZED,
+                                is_maximized);
     }
 }
 
@@ -1276,9 +1246,12 @@ caja_navigation_window_class_init (CajaNavigationWindowClass *class)
     CAJA_WINDOW_CLASS (class)->bookmarks_placeholder = MENU_PATH_BOOKMARKS_PLACEHOLDER;
 
     G_OBJECT_CLASS (class)->finalize = caja_navigation_window_finalize;
+#if GTK_CHECK_VERSION (3, 0, 0)
+    GTK_WIDGET_CLASS (class)->destroy = caja_navigation_window_destroy;
+#else
     GTK_OBJECT_CLASS (class)->destroy = caja_navigation_window_destroy;
+#endif
     GTK_WIDGET_CLASS (class)->show = caja_navigation_window_show;
-    GTK_WIDGET_CLASS (class)->unrealize = caja_navigation_window_unrealize;
     GTK_WIDGET_CLASS (class)->window_state_event = caja_navigation_window_state_event;
     GTK_WIDGET_CLASS (class)->key_press_event = caja_navigation_window_key_press_event;
     GTK_WIDGET_CLASS (class)->button_press_event = caja_navigation_window_button_press_event;
@@ -1295,18 +1268,20 @@ caja_navigation_window_class_init (CajaNavigationWindowClass *class)
 
     g_type_class_add_private (G_OBJECT_CLASS (class), sizeof (CajaNavigationWindowDetails));
 
+    g_signal_connect_swapped (caja_preferences,
+                              "changed::" CAJA_PREFERENCES_MOUSE_BACK_BUTTON,
+                              G_CALLBACK(mouse_back_button_changed),
+                              NULL);
 
-    eel_preferences_add_callback (CAJA_PREFERENCES_MOUSE_BACK_BUTTON,
-                                  mouse_back_button_changed,
-                                  NULL);
+    g_signal_connect_swapped (caja_preferences,
+                              "changed::" CAJA_PREFERENCES_MOUSE_FORWARD_BUTTON,
+                              G_CALLBACK(mouse_forward_button_changed),
+                              NULL);
 
-    eel_preferences_add_callback (CAJA_PREFERENCES_MOUSE_FORWARD_BUTTON,
-                                  mouse_forward_button_changed,
-                                  NULL);
-
-    eel_preferences_add_callback (CAJA_PREFERENCES_MOUSE_USE_EXTRA_BUTTONS,
-                                  use_extra_mouse_buttons_changed,
-                                  NULL);
+    g_signal_connect_swapped (caja_preferences,
+                              "changed::" CAJA_PREFERENCES_MOUSE_USE_EXTRA_BUTTONS,
+                              G_CALLBACK(use_extra_mouse_buttons_changed),
+                              NULL);
 }
 
 static CajaWindowSlot *
