@@ -16,7 +16,7 @@
  *
  *  You should have received a copy of the GNU General Public License
  *  along with this program; if not, write to the Free Software
- *  Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
+ *  Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1301, USA.
  *
  *  Author: Dave Camp <dave@ximian.com>
  */
@@ -30,6 +30,10 @@
 #include <gtk/gtk.h>
 #include <glib/gi18n.h>
 
+#if !GTK_CHECK_VERSION(3,0,0)
+#define gtk_widget_get_preferred_size(x,y,z) gtk_widget_size_request(x,y)
+#endif
+
 typedef struct
 {
     char *title;
@@ -146,7 +150,7 @@ caja_side_pane_size_allocate (GtkWidget *widget,
     frame = pane->details->title_frame;
     hbox = pane->details->title_hbox;
 
-    gtk_widget_get_child_requisition (hbox, &child_requisition);
+    gtk_widget_get_preferred_size (hbox, &child_requisition, NULL);
     width = child_requisition.width;
 
     gtk_widget_get_allocation (frame, &frame_allocation);
@@ -249,7 +253,7 @@ select_button_press_callback (GtkWidget *widget,
         gtk_widget_get_allocation (widget, &allocation);
         width = allocation.width;
         gtk_widget_set_size_request (side_pane->details->menu, -1, -1);
-        gtk_widget_size_request (side_pane->details->menu, &requisition);
+        gtk_widget_get_preferred_size (side_pane->details->menu, &requisition, NULL);
         gtk_widget_set_size_request (side_pane->details->menu,
                                      MAX (width, requisition.width), -1);
 
@@ -274,10 +278,10 @@ select_button_key_press_callback (GtkWidget *widget,
 
     side_pane = CAJA_SIDE_PANE (user_data);
 
-    if (event->keyval == GDK_space ||
-            event->keyval == GDK_KP_Space ||
-            event->keyval == GDK_Return ||
-            event->keyval == GDK_KP_Enter)
+    if (event->keyval == GDK_KEY_space ||
+        event->keyval == GDK_KEY_KP_Space ||
+        event->keyval == GDK_KEY_Return ||
+        event->keyval == GDK_KEY_KP_Enter)
     {
         gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (widget), TRUE);
         gtk_menu_popup (GTK_MENU (side_pane->details->menu),
