@@ -17,7 +17,7 @@
  *
  * You should have received a copy of the GNU General Public License
  * along with this program; if not, write to the Free Software
- * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
+ * Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1301, USA.
  *
  * Author: John Sullivan <sullivan@eazel.com>
  */
@@ -43,14 +43,12 @@
 #include <gio/gio.h>
 #include <glib/gi18n.h>
 #include <eel/eel-gtk-extensions.h>
-#include <eel/eel-preferences.h>
 #include <libcaja-extension/caja-menu-provider.h>
 #include <libcaja-private/caja-file-utilities.h>
 #include <libcaja-private/caja-global-preferences.h>
 #include <libcaja-private/caja-icon-names.h>
 #include <libcaja-private/caja-ui-utilities.h>
 #include <libcaja-private/caja-module.h>
-#include <libcaja-private/caja-undo-manager.h>
 #include <libcaja-private/caja-search-directory.h>
 #include <libcaja-private/caja-search-engine.h>
 #include <libcaja-private/caja-signaller.h>
@@ -245,17 +243,9 @@ action_connect_to_server_callback (GtkAction *action,
                                    gpointer user_data)
 {
     CajaWindow *window = CAJA_WINDOW (user_data);
-    CajaWindowSlot *slot;
     GtkWidget *dialog;
-    GFile *location;
 
-    slot = caja_window_get_active_slot (window);
-    location = caja_window_slot_get_location (slot);
-    dialog = caja_connect_server_dialog_new (window, location);
-    if (location)
-    {
-        g_object_unref (location);
-    }
+    dialog = caja_connect_server_dialog_new (window);
 
     gtk_widget_show (dialog);
 }
@@ -273,16 +263,6 @@ action_stop_callback (GtkAction *action,
     caja_window_slot_stop_loading (slot);
 }
 
-#ifdef TEXT_CHANGE_UNDO
-static void
-action_undo_callback (GtkAction *action,
-                      gpointer user_data)
-{
-    caja_undo_manager_undo
-    (CAJA_WINDOW (user_data)->application->undo_manager);
-}
-#endif
-
 static void
 action_home_callback (GtkAction *action,
                       gpointer user_data)
@@ -437,7 +417,7 @@ show_hidden_files_preference_callback (gpointer callback_data)
         /* update button */
         g_signal_handlers_block_by_func (action, action_show_hidden_files_callback, window);
         gtk_toggle_action_set_active (GTK_TOGGLE_ACTION (action),
-                                      eel_preferences_get_boolean (CAJA_PREFERENCES_SHOW_HIDDEN_FILES));
+                                      g_settings_get_boolean (caja_preferences, CAJA_PREFERENCES_SHOW_HIDDEN_FILES));
         g_signal_handlers_unblock_by_func (action, action_show_hidden_files_callback, window);
 
         /* inform views */
@@ -520,11 +500,12 @@ action_about_caja_callback (GtkAction *action,
         "Perberos",
         "Steve Zesch",
         "Stefano Karapetsas",
+        "Jasmine Hassan",
         NULL
     };
     const gchar *documenters[] =
     {
-        "MATE Documentation Team",
+        "GNOME Documentation Team",
         "Sun Microsystem",
         NULL
     };
@@ -554,7 +535,7 @@ action_about_caja_callback (GtkAction *action,
                                          "files and folders, both on "
                                          "your computer and online."),
                            "copyright", _("Copyright \xC2\xA9 1999-2009 The Nautilus authors\n"
-                                          "Copyright \xC2\xA9 2011 The Caja authors"),
+                                          "Copyright \xC2\xA9 2011-2013 The Caja authors"),
                            "license", license_trans,
                            "wrap-license", TRUE,
                            "authors", authors,
@@ -853,12 +834,6 @@ static const GtkActionEntry main_entries[] =
         NULL, N_("Edit Caja preferences"),
         G_CALLBACK (action_preferences_callback)
     },
-    #ifdef TEXT_CHANGE_UNDO
-    /* name, stock id, label */  { "Undo", NULL, N_("_Undo"),
-        "<control>Z", N_("Undo the last text change"),
-        G_CALLBACK (action_undo_callback)
-    },
-    #endif
     /* name, stock id, label */  { "Up", GTK_STOCK_GO_UP, N_("Open _Parent"),
         "<alt>Up", N_("Open the parent folder"),
         G_CALLBACK (action_up_callback)
@@ -992,13 +967,13 @@ caja_window_initialize_menus (CajaWindow *window)
     action = gtk_action_group_get_action (action_group, CAJA_ACTION_SHOW_HIDDEN_FILES);
     g_signal_handlers_block_by_func (action, action_show_hidden_files_callback, window);
     gtk_toggle_action_set_active (GTK_TOGGLE_ACTION (action),
-                                  eel_preferences_get_boolean (CAJA_PREFERENCES_SHOW_HIDDEN_FILES));
+                                  g_settings_get_boolean (caja_preferences, CAJA_PREFERENCES_SHOW_HIDDEN_FILES));
     g_signal_handlers_unblock_by_func (action, action_show_hidden_files_callback, window);
 
 
-    eel_preferences_add_callback_while_alive (CAJA_PREFERENCES_SHOW_HIDDEN_FILES,
-            show_hidden_files_preference_callback,
-            window, G_OBJECT (window));
+    g_signal_connect_swapped (caja_preferences, "changed::" CAJA_PREFERENCES_SHOW_HIDDEN_FILES,
+                              G_CALLBACK(show_hidden_files_preference_callback),
+                              window);
 
     window->details->ui_manager = gtk_ui_manager_new ();
     ui_manager = window->details->ui_manager;
