@@ -16,8 +16,8 @@
 
    You should have received a copy of the GNU General Public
    License along with this program; if not, write to the
-   Free Software Foundation, Inc., 59 Temple Place - Suite 330,
-   Boston, MA 02111-1307, USA.
+   Free Software Foundation, Inc., 51 Franklin St, Fifth Floor,
+   Boston, MA 02110-1301, USA.
 
    Author: Christian Neumair <cneumair@gnome.org>
 */
@@ -33,6 +33,8 @@
 #include <eel/eel-gtk-macros.h>
 #include <eel/eel-string.h>
 
+#include "glibcompat.h" /* for g_list_free_full */
+
 static void caja_window_slot_init       (CajaWindowSlot *slot);
 static void caja_window_slot_class_init (CajaWindowSlotClass *class);
 static void caja_window_slot_dispose    (GObject *object);
@@ -662,7 +664,7 @@ caja_window_slot_dispose (GObject *object)
         g_object_ref (slot->location);
     }
 
-    eel_g_list_free_deep (slot->pending_selection);
+    g_list_free_full (slot->pending_selection, g_free);
     slot->pending_selection = NULL;
 
     if (slot->current_location_bookmark != NULL)
