@@ -17,8 +17,8 @@
  *
  * You should have received a copy of the GNU General Public
  * License along with this program; if not, write to the
- * Free Software Foundation, Inc., 59 Temple Place - Suite 330,
- * Boston, MA 02111-1307, USA.
+ * Free Software Foundation, Inc., 51 Franklin St, Fifth Floor,
+ * Boston, MA 02110-1301, USA.
  *
  * Authors:
  *       Maciej Stachowiak <mjs@eazel.com>
@@ -35,11 +35,7 @@
 #include "fm-tree-model.h"
 #include "fm-properties-window.h"
 #include <string.h>
-#include <eel/eel-alert-dialog.h>
-#include <eel/eel-glib-extensions.h>
 #include <eel/eel-gtk-extensions.h>
-#include <eel/eel-preferences.h>
-#include <eel/eel-stock-dialogs.h>
 #include <gtk/gtk.h>
 #include <glib/gi18n.h>
 #include <gio/gio.h>
@@ -54,12 +50,13 @@
 #include <libcaja-private/caja-icon-names.h>
 #include <libcaja-private/caja-program-choosing.h>
 #include <libcaja-private/caja-tree-view-drag-dest.h>
-#include <libcaja-private/caja-cell-renderer-pixbuf-emblem.h>
 #include <libcaja-private/caja-sidebar-provider.h>
 #include <libcaja-private/caja-module.h>
 #include <libcaja-private/caja-window-info.h>
 #include <libcaja-private/caja-window-slot-info.h>
 
+#include <src/glibcompat.h> /* for g_list_free_full */
+
 typedef struct
 {
     GObject parent;
@@ -118,7 +115,6 @@ typedef struct
 } PrependURIParameters;
 
 static GdkAtom copied_files_atom;
-static gboolean show_delete_command_auto_value;
 
 static void  fm_tree_view_iface_init        (CajaSidebarIface         *iface);
 static void  sidebar_provider_iface_init    (CajaSidebarProviderIface *iface);
@@ -621,10 +617,6 @@ static char *
 get_root_uri_callback (CajaTreeViewDragDest *dest,
                        gpointer user_data)
 {
-    FMTreeView *view;
-
-    view = FM_TREE_VIEW (user_data);
-
     /* Don't allow drops on background */
     return NULL;
 }
@@ -811,7 +803,7 @@ button_pressed_callback (GtkTreeView *treeview, GdkEventButton *event,
         can_move_file_to_trash = caja_file_can_trash (view->details->popup_file);
         gtk_widget_set_sensitive (view->details->popup_trash, can_move_file_to_trash);
 
-        if (show_delete_command_auto_value)
+        if (g_settings_get_boolean (caja_preferences, CAJA_PREFERENCES_ENABLE_DELETE))
         {
             parent_file_is_writable = is_parent_writable (view->details->popup_file);
             file_is_home_or_desktop = caja_file_is_home (view->details->popup_file)
@@ -1065,7 +1057,7 @@ paste_clipboard_data (FMTreeView *view,
             gtk_clipboard_clear (caja_clipboard_get (GTK_WIDGET (view)));
         }
 
-        eel_g_list_free_deep (item_uris);
+    	g_list_free_full (item_uris, g_free);
     }
 }
 
@@ -1128,7 +1120,7 @@ fm_tree_view_trash_cb (GtkWidget *menu_item,
     caja_file_operations_trash_or_delete (list,
                                           fm_tree_view_get_containing_window (view),
                                           NULL, NULL);
-    eel_g_object_list_free (list);
+    g_list_free_full (list, g_free);
 }
 
 static void
@@ -1137,7 +1129,7 @@ fm_tree_view_delete_cb (GtkWidget *menu_item,
 {
     GList *location_list;
 
-    if (!show_delete_command_auto_value)
+    if (!g_settings_get_boolean (caja_preferences, CAJA_PREFERENCES_ENABLE_DELETE))
     {
         return;
     }
@@ -1146,7 +1138,7 @@ fm_tree_view_delete_cb (GtkWidget *menu_item,
                                     caja_file_get_location (view->details->popup_file));
 
     caja_file_operations_delete (location_list, fm_tree_view_get_containing_window (view), NULL, NULL);
-    eel_g_object_list_free (location_list);
+    g_list_free_full (location_list, g_object_unref);
 }
 
 static void
@@ -1477,13 +1469,12 @@ create_tree (FMTreeView *view)
     /* Create column */
     column = gtk_tree_view_column_new ();
 
-    cell = caja_cell_renderer_pixbuf_emblem_new ();
+    cell = gtk_cell_renderer_pixbuf_new ();
     gtk_tree_view_column_pack_start (column, cell, FALSE);
     gtk_tree_view_column_set_attributes (column, cell,
                                          "pixbuf", FM_TREE_MODEL_CLOSED_PIXBUF_COLUMN,
                                          "pixbuf_expander_closed", FM_TREE_MODEL_CLOSED_PIXBUF_COLUMN,
                                          "pixbuf_expander_open", FM_TREE_MODEL_OPEN_PIXBUF_COLUMN,
-                                         "pixbuf_emblem", FM_TREE_MODEL_EMBLEM_PIXBUF_COLUMN,
                                          NULL);
 
     cell = gtk_cell_renderer_text_new ();
@@ -1533,7 +1524,7 @@ update_filtering_from_preferences (FMTreeView *view)
     {
         fm_tree_model_set_show_hidden_files
         (view->details->child_model,
-         eel_preferences_get_boolean (CAJA_PREFERENCES_SHOW_HIDDEN_FILES));
+         g_settings_get_boolean (caja_preferences, CAJA_PREFERENCES_SHOW_HIDDEN_FILES));
     }
     else
     {
@@ -1543,7 +1534,7 @@ update_filtering_from_preferences (FMTreeView *view)
     }
     fm_tree_model_set_show_only_directories
     (view->details->child_model,
-     eel_preferences_get_boolean (CAJA_PREFERENCES_TREE_SHOW_ONLY_DIRECTORIES));
+     g_settings_get_boolean (caja_tree_sidebar_preferences, CAJA_PREFERENCES_TREE_SHOW_ONLY_DIRECTORIES));
 }
 
 static void
@@ -1600,13 +1591,13 @@ fm_tree_view_init (FMTreeView *view)
 
     view->details->selecting = FALSE;
 
-    eel_preferences_add_callback_while_alive (CAJA_PREFERENCES_SHOW_HIDDEN_FILES,
-            filtering_changed_callback, view, G_OBJECT (view));
-    eel_preferences_add_callback_while_alive (CAJA_PREFERENCES_SHOW_BACKUP_FILES,
-            filtering_changed_callback, view, G_OBJECT (view));
-    eel_preferences_add_callback_while_alive (CAJA_PREFERENCES_TREE_SHOW_ONLY_DIRECTORIES,
-            filtering_changed_callback, view, G_OBJECT (view));
-
+    g_signal_connect_swapped (caja_preferences,
+                              "changed::" CAJA_PREFERENCES_SHOW_HIDDEN_FILES,
+                              G_CALLBACK(filtering_changed_callback),
+                              view);
+    g_signal_connect_swapped (caja_tree_sidebar_preferences,
+                              "changed::" CAJA_PREFERENCES_TREE_SHOW_ONLY_DIRECTORIES,
+                              G_CALLBACK (filtering_changed_callback), view);
     view->details->popup_file = NULL;
 
     view->details->clipboard_handler_id =
@@ -1679,6 +1670,14 @@ fm_tree_view_dispose (GObject *object)
         view->details->volume_monitor = NULL;
     }
 
+    g_signal_handlers_disconnect_by_func (caja_preferences,
+                                          G_CALLBACK(filtering_changed_callback),
+                                          view);
+
+    g_signal_handlers_disconnect_by_func (caja_tree_sidebar_preferences,
+                                          G_CALLBACK(filtering_changed_callback),
+                                          view);
+
     view->details->window = NULL;
 
     G_OBJECT_CLASS (parent_class)->dispose (object);
@@ -1703,9 +1702,6 @@ fm_tree_view_class_init (FMTreeViewClass *class)
     G_OBJECT_CLASS (class)->finalize = fm_tree_view_finalize;
 
     copied_files_atom = gdk_atom_intern ("x-special/mate-copied-files", FALSE);
-
-    eel_preferences_add_auto_boolean (CAJA_PREFERENCES_ENABLE_DELETE,
-                                      &show_delete_command_auto_value);
 }
 
 static const char *
